//-----------------------------------------------------------------------------
// File: IslandTacticsGuard.h
//
// Desc: Tactic manager for one game island
//
// Copyright (c) 2000-2011 Black Element Software. All rights reserved
//-----------------------------------------------------------------------------

/** \addtogroup MissionsAPI */
/**@{*/
//! Guard component
class GuardTactic extends TacticCompoment
{
	bool Inizialized;
	bool UnitsSended;					// flag, zdali byly poslany jednotky do akce
	bool TacticGotHealer;

	float Time;							// timer
	float TimeUpd;						//timer pro update enemeka a vlastnich jednotek

	const float OffTime = 60;		// Konstanta, jak dlouho muze byt ma jednotka mimo trigger

	// Force system
	int MyForce[3];						// AA, AT, AL
	int MyWalrusDefense;
	int MyMantaDefense;
	int EnemyForce[3];					// AA, AT, AL
	int EnemyWalrusDefense;
	int EnemyMantaDefense;

	vector GoPosition;					// Pozice, kam maji walrusy vyject

	int UnitsCnt;						// MyUnitsCnt;
	ObjectHandlerBase Units[32];		// tactic units array
	float TimeOn[32];					// tactic units: last time in the trigger
	vector BasePos[32];					// tactic units: base position
	
	int EnemiesCnt;						// Pocet nepratel
	Object Enemies[32];					// Pole nepratel
	int OutsideAttacker[32];			// zdali je attacker mimo trigger

	// Triggery ..... zjistil jsem, ze si vystacime s jednim triggerem pro taktiku.
	// Vice nema smysl. Nabylo by to dost na slozitosti.
	Trigger ChildTriggers[8];
	int ChildTrigCnt;
	int Inside;								// Enemy jednotky v triggeru
	
	// Hlidane objekty Guard taktiky
	int GuardedObjCnt;
	Object GuardedObjects[32];

	// routes
	int RouteCnt;							// pocet rout
	int RouteStatus[16];					// route unit status
	RouteInst Routes[16];					// Routy

	
	//string WidgetMsg;


	//--------------------------------------------------------------------------
	//! ctor
	//--------------------------------------------------------------------------
	void GuardTactic()
	{
		IsValid = true;

		for( int i = 0; i < 32; i++ )
		{
			TimeOn[i] = -1;
			OutsideAttacker[i] = -1;
		}
	}

	//--------------------------------------------------------------------------
	//! callback from C++
	//--------------------------------------------------------------------------
	void SetLimits( int min, int opt, int cap )
	{

	}
	
	//--------------------------------------------------------------------------
	//! callback from C++
	//--------------------------------------------------------------------------
	void AssignRoute( RouteInst route )
	{
		Routes[RouteCnt] = route;
		RouteCnt++;
	}

	//--------------------------------------------------------------------------
	//! event called on object assign
	//--------------------------------------------------------------------------
	void OnAssignObject( ObjectHandlerBase object )
	{

		if(UnitsCnt >= 32)
			return;
			
		//STUB: must check for doubled objects :|
		for( int i = 0; i < UnitsCnt; i++ )
		{
			ObjectHandlerBase ooo = Units[i];
			if ( ooo == object )
				return;
		}

#ifdef DEVELOPER
		if ( TACTICS_DEBUG )
		{
    		string txt = "[" + GetName() + "]" + " >>> Guard prijal (OnAssignObject() ) object:";
    		Print( txt );
    		Print( object );
		}
#endif
		Object objects[32];
		int objectCount = object.BuildObjectList( objects );
		if( objectCount == 0 ) return;
		
		// debug promenne
		int mantaDefense = MyMantaDefense;
		int walrusDefense = MyWalrusDefense;
		
		// Vypocet force a defense assignovane jednotky
		for( int ii = 0; ii < objectCount; ii++ )
		{
			Object o = objects[ii];
			InteractiveEntity ent = o.GetEntity();
			RouteInst r = NULL;
				
			int type = ent.GetAttackType();

			if( type == ATTACK_NONE  )
			{
#ifdef DEVELOPER
				if ( ent.IsHealer() == false )
				{
					if ( TACTICS_DEBUG )
					{
						txt = "[" + GetName() + "]" + " >>> : OnAssignObject - Jednotka beze zbrane!" ;
						Print( txt );
						continue;
					}
				}
#endif
			}
			else
			{
				MyForce[type] = MyForce[type] + ent.GetAttackPower();
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					txt = "[" + GetName() + "]" + " >>> " ;
					switch( type )
					{
						case ATTACK_AA:
							txt += " Jednotka protiletecka.";
							break;

						case ATTACK_AT:
							txt += " Jednotka protipozemni.";
							break;

						case ATTACK_AL:
							txt += " Jednotka protivsemu.";
							break;
					
					}
					Print( txt );
				}
#endif
			}
			
			if ( o.GetType() == AI_MANTA )
			{
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					txt = "[" + GetName() + "]" + " >>> prijal mantu.";
					Print(txt);
				}
#endif
			 	o.SetStance( STANCE_DEFENSIVE );
			 	
				MyMantaDefense += ent.GetDefense();

				// Poslani Manty na routu, pokud to dovoluje
				for( int k = 0; k < RouteCnt; k++ )
				{
					r = Routes[k];
					if ( r == NULL ) break;
					if ( r.GetUnitFilter() == 2 ) continue;
					if ( r.GetCapacity() > RouteStatus[k] )
					{
#ifdef DEVELOPER
						if ( TACTICS_DEBUG )
						{
							txt = "[" + GetName() + "]" + " >>> a poslal ji na routu.";
							Print(txt);
						}
#endif							
						object.PlanAction( CreateRouteFollowAction( r, true , false, r.GetFirstNode(), r.GetFirstNode() ), false, false );
						RouteStatus[k] = RouteStatus[k] + 1;
						break;
					}
				}
			}
			
			if( o.GetType() == AI_WALRUS )
			{
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					txt = "[" + GetName() + "]" + " >>> prijal walrus.";
					Print(txt);	
				}
#endif
			 	o.SetStance( STANCE_DEFENSIVE );
				MyWalrusDefense += ent.GetDefense();
				
				if( ent.IsHealer() )
				{
					MyWalrusDefense += 2;
					MyMantaDefense += 2;
					TacticGotHealer = true;
				}
				
				// Poslani walruse na routu, pokud to dovoluje
				for( k = 0; k < RouteCnt; k++ )
				{
					r = Routes[k];
					if ( r == NULL ) break;
					if ( r.GetUnitFilter() == 1 ) continue;
					if ( r.GetCapacity() > RouteStatus[k] )
					{
						object.PlanAction( CreateRouteFollowAction( r, true , false, r.GetFirstNode(), r.GetFirstNode() ), false, false );
						RouteStatus[k] = RouteStatus[k] + 1;
						break;
					}
				}
			}
			
			if( o.GetType() == AI_SOLDIER )
			{
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					txt = "[GUARD] " + GetName() + " >>> prijal character.";
					Print(txt);	
				}
#endif
			 	o.SetStance( STANCE_DEFENSIVE );

				// Poslani Soldiera na routu, pokud to dovoluje
				for( k = 0; k < RouteCnt; k++ )
				{
					r = Routes[k];
					if ( r == NULL ) break;
					if ( r.GetUnitFilter() == 1 ) continue;
					if ( r.GetCapacity() > RouteStatus[k] )
					{
						object.PlanAction( CreateRouteFollowAction( r, true , false, r.GetFirstNode(), r.GetFirstNode() ), false, false );
						RouteStatus[k] = RouteStatus[k] + 1;
						break;
					}
				}
			}
		}
		
		// Ulozeni jednotky do pole v taktice
		Units[UnitsCnt] = object;
		BasePos[UnitsCnt] = object.GetPosition();
		TimeOn[UnitsCnt] = -1;
		UnitsCnt++;
		
		// poslani reinforcement jednotky rovnou do akce. Reinf. jednotka se pozna, ze byla assignovana v dobe akce
		if ( EnemiesCnt > 0 )
		{
			int defense = MyWalrusDefense + MyMantaDefense - mantaDefense - walrusDefense;

#ifdef DEVELOPER
			if ( TACTICS_DEBUG )
			{
			    	string str = "[" + GetName() + "]" + " >>> : Unit reinforcement with defence: " + itoa( defense );
		    	 	Print( str );
			}
#endif
			if ( GoPosition != ZeroVec )
				object.PlanAction( CreateMovementAction( GoPosition ), false, true );
			else
			{
				Trigger t = ChildTriggers[0];

				// Pokud neni trigger, reinforcement jednotky se nedostanou na misto!!!
				if ( t != NULL && object != NULL )
					object.PlanAction( CreateMovementAction( t.GetPosition() ), false, true );
			}
		}
		
	}
	
	//--------------------------------------------------------------------------
	//! event called on object removal
	//--------------------------------------------------------------------------
	void OnRemoveObject( ObjectHandlerBase object )
	{
		// Odstraneni z pole
		for( int j = 0; j < UnitsCnt; j++ )
		{
			ObjectHandlerBase temp = Units[j];
			if( temp == object )
			{
				UnitsCnt--;
				if(j != UnitsCnt)
				{
					Units[j] = Units[UnitsCnt];
					BasePos[j] = BasePos[UnitsCnt];
					TimeOn[j] = TimeOn[UnitsCnt];
				}
				Units[UnitsCnt] = NULL;
				break;
			}
		}

		Object objects[32];
		int objectCount = object.BuildObjectList( objects );

		if( objectCount == 0 )
			return;

		// Odecet sily a defense jednotky
		for( int i = 0; i < objectCount; i++ )
		{
			Object o = objects[i];
			InteractiveEntity ent = o.GetEntity();
			int type = ent.GetAttackType();
			
			if ( GoHealer && GoHealer == ent )
			    GoHealer = NULL;

			if( type == ATTACK_NONE )
			{
			    if ( ent.IsHealer() == false )
			    {
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
					    	string str = "[" + GetName() + "]" + " >>> : OnRemoveObject - Nevalidni jednotka - ale co to znamena???";
				    	 	Print( str );
					}
#endif
					continue;
				}
			}
			else
			{
				MyForce[type] = MyForce[type] - ent.GetAttackPower();
			}

			
			if ( o.GetType() == AI_MANTA )
				MyMantaDefense -= ent.GetDefense();

			if ( o.GetType() == AI_WALRUS )
			{
				if ( ent.IsHealer() == true )
				{
					MyWalrusDefense -= 2;
					MyMantaDefense -= 2;
				}

				MyWalrusDefense -= ent.GetDefense();
			}
		}
	}
	
	//--------------------------------------------------------------------------
	//! event called when new child trigger is assigned to be guarded
	//--------------------------------------------------------------------------
	void RegisterChildTrigger( Trigger t )
	{
		ChildTriggers[ChildTrigCnt] = t;
		ChildTrigCnt++;
	}

	//--------------------------------------------------------------------------
	// Pridani Enemaka do pole enemaku, odebiram jak silu, tak defense
	//--------------------------------------------------------------------------
	bool AddEnemy( Object o )
	{
		//pokud je to ostrovni jednotka, tak neni enemy pro taktiku
		CCPlayer islandPlayer = g_Game.GetCurrentIsland().GetPlayer();
		if ( o.GetPlayer().GetName() == islandPlayer.GetName() )
			return false;

		if(EnemiesCnt >= 32)
			return false;

	    // Zde je rovnou check, zdali je jednotka uz v poli enemaku
		for( int i = 0; i < EnemiesCnt; i++ )
		{
			if ( Enemies[i] == o )
				return false;
		}

	    // Print( "Pridan Enemak" );
		InteractiveEntity ent = o.GetEntity();
		
		if ( ent == NULL )
			return false;
				
		Enemies[i] = o;

		// Opet je to jen pro jeden trigger - rika, zdali to je utocnik mimo trigger
		Trigger t = ChildTriggers[0];
		if ( t != NULL )
		{
			if ( t.Inside( o.GetPosition() ) )
				OutsideAttacker[i] = 0;
			else
				OutsideAttacker[i] = 1;
		}

		int type = ent.GetAttackType();
		
		if ( type == ATTACK_NONE )
		{
			if ( ent.IsHealer() == false )
			{
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					string txt = "Guard " + GetName() + " >>> Komponenta zaznamenala jednotku je vsak beze zbrane!"
					Print( txt );
				}
#endif
				return false;
			}
		}
		else
			EnemyForce[type] = EnemyForce[type] + ent.GetAttackPower();

		type = o.GetType();

		if ( type ==  AI_WALRUS )
		{
			if( ent.IsHealer() == true )
			{
				EnemyWalrusDefense += 2;
				EnemyMantaDefense += 2;
			}
			EnemyWalrusDefense += ent.GetDefense();
		}

		if ( type == AI_MANTA )
			EnemyMantaDefense += ent.GetDefense();

		EnemiesCnt++;
		return true;
		
	}
	//--------------------------------------------------------------------------
	//! child trigger activated
	//--------------------------------------------------------------------------
	void OnChildTriggerEntered( Trigger t, Object o )
	{
#ifdef DEVELOPER
		if ( EnemiesCnt == 0 )
		{
			if ( TACTICS_DEBUG )
			{
				string str = "[" + GetName() + "]"  + " Enemy Entered trigger!!!, Walrus defense: " + itoa( EnemyWalrusDefense ) + ", Manta defense: " + itoa( EnemyMantaDefense );
				Print( str );
			}
		}
#endif	    
		int type = 0;
		AddEnemy( o );
		Inside++;
		
		// Nastaveni pozice, kam se maji jednotky vydat
		if ( GoPosition == ZeroVec )
			GoPosition = t.GetPosition();
	}
	
	//--------------------------------------------------------------------------
	// Odstranuji jednotku z pole enemaku, odebiram jeji silu i defense
	//--------------------------------------------------------------------------
	void EraseEnemy( Object o )
	{
		for( int i = 0; i < EnemiesCnt; i++ )
		{
			if ( Enemies[i] != o )
				continue;
				
			EnemiesCnt--;
			if (i != EnemiesCnt)
			{
				Enemies[i] = Enemies[EnemiesCnt]
				OutsideAttacker[i] = OutsideAttacker[EnemiesCnt];
			}

			Enemies[EnemiesCnt] = NULL;
			OutsideAttacker[EnemiesCnt] = 0; 
			
			InteractiveEntity ent = o.GetEntity();

			int type = ent.GetAttackType();
			// utok
			if ( type == ATTACK_NONE )
			{
#ifdef DEVELOPER
				if ( !ent.IsHealer() )
				{
					if ( TACTICS_DEBUG )
						Print( "GuardLeft: Jednotka beze zbrane! ");
				}
#endif
			}
			else
			{
				EnemyForce[type] = EnemyForce[type] - ent.GetAttackPower();
				if ( EnemyForce[type] < 0 )
					EnemyForce[type] = 0;
			}

			type = o.GetType();

			if ( type ==  AI_WALRUS )
			{
				if ( ent.IsHealer() == true )
				{
					EnemyWalrusDefense -= 2;
					EnemyMantaDefense -= 2;

					if ( EnemyMantaDefense < 0 )
						EnemyMantaDefense = 0;
					if ( EnemyWalrusDefense < 0 )
						EnemyWalrusDefense = 0;
				}

				EnemyWalrusDefense -= ent.GetDefense();
			}

			if ( type == AI_MANTA )
			{
				EnemyMantaDefense -= ent.GetDefense();
				if ( EnemyMantaDefense < 0 )
					EnemyMantaDefense = 0;	
			}

			break;
		}
 	}
 	
	//--------------------------------------------------------------------------
	//! child trigger left
	//--------------------------------------------------------------------------
	void OnChildTriggerLeft( Trigger t, Object o )
	{
#ifdef DEVELOPER
		if ( TACTICS_DEBUG ) 
		{
			string str = "[" + GetName() + "]" + " >>> Nejaka jednotka opustila trigger (umrela, nebo ho jen opustila) - odeber silu.";
			Print( str );
		}
#endif
		EraseEnemy( o );
		Inside--;
	}
	
	//--------------------------------------------------------------------------
	//! guarded object attacked
	//--------------------------------------------------------------------------
	void OnGuardedObjectAttacked( Object o )
	{
		//TEMP: just add to enemy array permanently
#ifdef DEVELOPER
		if ( !UnitsSended )
		{
			if( TACTICS_DEBUG ) 
			{
				string str = "[" + GetName() + "]"  + " >>> Guarded building under attack! Walrus defense: " + itoa( EnemyWalrusDefense ) + ", Manta defense: " + itoa( EnemyMantaDefense );
				Print( str );
			}
		}
#endif
		Object temp[32];
		// Zjisti attackery na objektu na ktery bylo zautoceno
		int tempCnt = o.GetAttackers( temp );
	
		// Projdu pole attackeru a pridam je do pole enemaku
		for( int i = 0; i < tempCnt; i++ )
		{
			Object obj = temp[i];
			// Tady lezou attackeri, ktere jsou i NULL!!!!
			InteractiveEntity ent = obj.GetEntity();
			
			if ( ent == NULL )
			{
#ifdef DEVELOPER
				Print( obj.GetName() );
				Print( ClassName( obj ) );
#endif
				continue;
			}

			AddEnemy( obj );
			
			// Print( i );
			// Print( ent );
		}
	}
	
	//--------------------------------------------------------------------------
	// Init
	//--------------------------------------------------------------------------
	bool Init()
	{
		for( int i = 0; i < UnitsCnt; i++ )
		{
			ObjectHandlerBase ohb = Units[i];
			BasePos[i] = ohb.GetPosition();
		}
		
		return true;
	}
	
	//--------------------------------------------------------------------------
	// Update my Units
	//--------------------------------------------------------------------------
	void UpdateMyUnits()
	{
	    // Plati pro jeden trigger.... pokud je vice triggeru, musi upravit update pro vice triggerovou zalezitost
	    // Pokud je jednotka delsi dobu mimo vlastni trigger po zahajeni akce, tak ji vracim na puvodni pozici
		Trigger t = ChildTriggers[0];
		ObjectHandlerBase unit;
		float actTime = g_Game.GetCampaignTime();
		float divTime = 0.0;
		string str;

		// Vlastni jednotky
		for( int i = 0; i < UnitsCnt; i++ )
		{
			unit = Units[i];

			if ( Inside > 0 )
				unit.SetStance( STANCE_AGGRESSIVE );

			if ( t == NULL ) continue;
			
			// Je ma jednotka v triggeru
			if ( t.Inside( unit.GetPosition() ) )
				TimeOn[i] = actTime;
			else
			{
				vector bp = BasePos[i];
				if ( t.Inside( bp ) )
				{	
					divTime = actTime - TimeOn[i];
	
					// Jednotka delsi dobu mimo trigger! Posilam zpet
					if ( divTime > OffTime && !unit.HasActions() )
					{
#ifdef DEVELOPER
						if( TACTICS_DEBUG )
						{
							str = "[" + GetName() + "]"  + " >>> ( UpdateMyUnits() ) Jednotka je delsi cas mimo sve uzemi a nema plan -posleme ji zpet."
							Print( str );
							Print( unit );
						}
#endif
						unit.SetStance( STANCE_DEFENSIVE );
						unit.PlanAction( CreateMovementAction( BasePos[i] ), false, true );
					}
				}
				// Jednotky mimo trigger si neberou dalsi enemaky :)
				continue;
			}

			//-----------------------------------------------------------
			Object attackers[32];
			int attackersCnt = unit.GetAttackers( attackers );
			
			if ( attackersCnt > 0 )
				unit.SetStance( STANCE_AGGRESSIVE );
			
			// Utocnici na me jednotky
			for( int j = 0; j < attackersCnt; j++ )
			{
				Object attacker = attackers[j];
				bool exist = false;
				
				// Print( attackersCnt );
				// Pole enemaku
				for( int k = 0; k < EnemiesCnt; k++ )
				{
					if( attacker == Enemies[k] )
					{
						exist = true;
						break;
					}
				}
				
				if ( !exist )
				{
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]"  + " >>>  Novy attacker!: " + attacker.GetName() ;
						Print( str );
					}
#endif
					AddEnemy( attacker );
				}
			}
		}

#ifdef DEVELOPER
		if ( TACTICS_DEBUG )
		{
			str = "[" + GetName() + "]"  + " >>> ( UpdateMyUnits() ) sumar: ";
			Print( str );
			Print( UnitsCnt );
			Print( EnemiesCnt );
		}
#endif
	}

	//--------------------------------------------------------------------------
	// UpdateEnemy array
	//--------------------------------------------------------------------------
	void UpdateEnemies()
	{
   		// Vynulovani parametru
		EnemyForce[ATTACK_AA] = 0;
		EnemyForce[ATTACK_AT] = 0;
		EnemyForce[ATTACK_AL] = 0;

		EnemyWalrusDefense = 0;
		EnemyMantaDefense = 0;

		// Toto je provozuschopne pro jeden trigger... neni slozite to rozsirit
		Trigger t = ChildTriggers[0];

		for( int i = 0; i < EnemiesCnt; )
		{
			Object o = Enemies[i];
if ( !o )
break;
			// Pokud jednotka nezije, odstranim ji
			if( o && !o.IsAlive() )
			{
				EnemiesCnt--;
				if (i != EnemiesCnt)
				{
					Enemies[i] = Enemies[EnemiesCnt];
					OutsideAttacker[i] = OutsideAttacker[EnemiesCnt]; 
				}
				Enemies[EnemiesCnt] = NULL;
				OutsideAttacker[EnemiesCnt] = 0; 
		
				
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					string str = "[" + GetName() + "]"  + " >>>  Znicena jednotka nepritele... ";
					Print( str );
					Print( Enemies );
					Print( EnemiesCnt );
				}
#endif
				continue;
			}

			// pokud neni v triggeru a nema attackery, odstranuji jednotku
			if ( t != NULL && !t.Inside( o.GetPosition() ) )
			{
				// Attackeri
				int attackersCnt = o.GetNumAttackers();

				if( attackersCnt == 0 )
				{
					// Print("Jednotka is off!");
					EnemiesCnt--;
					if (i != EnemiesCnt)
					{
						Enemies[i] = Enemies[EnemiesCnt];
						OutsideAttacker[i] = OutsideAttacker[EnemiesCnt]; 
					}
					Enemies[EnemiesCnt] = NULL;
					OutsideAttacker[EnemiesCnt] = 0; 

					
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>>  Nepratelska jednotka mimo palebnou silu!";
						Print( str );
					}
#endif
					continue;
				}
				
				OutsideAttacker[i] = 1;
			}
			else
				OutsideAttacker[i] = 0;

			InteractiveEntity ent = o.GetEntity();

			int type = ent.GetAttackType();
			if ( type == ATTACK_NONE )
			{
				if ( !ent.IsHealer() )
				{
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]"  + " >>>  GuardUpdate: Jednotka beze zbrane! ";
						Print( str );
					}
#endif
					i++;
					continue;
				}
			}
			else
				EnemyForce[type] = EnemyForce[type] + ent.GetAttackPower();

			type = o.GetType();

			if ( type ==  AI_WALRUS )
			{
				if ( ent.IsHealer() == true )
				{
					EnemyWalrusDefense += 2;
					EnemyMantaDefense += 2;
				}

				EnemyWalrusDefense += ent.GetDefense();
			}

			if ( type == AI_MANTA )
				EnemyMantaDefense += ent.GetDefense();

			// Toto je provizorni situace pro utok Carrieru na spec. budovu.
			if ( type == AI_CARRIER )
			{
			    // Print("Carrier prudi");
				EnemyWalrusDefense += 10;
				EnemyMantaDefense += 10;
				EnemyForce[ATTACK_AA] = 12;
				EnemyForce[ATTACK_AT] = 3;
				EnemyForce[ATTACK_AL] = 3;
			}
			i++;
		}
	}
	
	//--------------------------------------------------------------------------
	// GoToPosition
	//--------------------------------------------------------------------------
	void GoToAction()
	{
	    int from = 0;
	    Object obj;

		for( int i = 0; i < UnitsCnt; i++ )
		{
			ObjectHandlerBase ohb = Units[i];

			if ( GoPosition != ZeroVec )
			{
				if ( ohb != NULL )
					ohb.PlanAction( CreateMovementAction( GoPosition ), true, true );
			}
			else
			{
			    for( int j = from; j < EnemiesCnt; j++ )
			    {
					if( OutsideAttacker[j] == 1 )
					{
						obj = Enemies[j];
						from = j + 1;
						break;
					}
				}
				
				if ( obj && ohb != NULL && ohb.GetType() == AI_MANTA && !ohb.HasActions() )
				{
					// Print( obj.GetName() );
					ohb.SetStance( STANCE_AGGRESSIVE );
					ohb.PlanAction( CreateAttackAction( obj ), true, true );
				}
			}
		}
	}
	
	//--------------------------------------------------------------------------
	// End action
	//--------------------------------------------------------------------------
	void EndAction()
	{
#ifdef DEVELOPER
		if ( TACTICS_DEBUG )
		{
			string str = "[" + GetName() + "]"  + " No enemies in area, no attack on the guarded building!";
			Print( str );
		}
#endif					
		if ( ReturnToBase )
		{
#ifdef DEVELOPER
			if ( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> ( EndAction() ) ReturnToBase je true, vratte se do baze.";
				Print( str );
			}
#endif
			for( int i = 0; i < UnitsCnt; i++ )
			{
				ObjectHandlerBase ooo = Units[i];
				vector base = BasePos[i];
				ooo.SetStance( STANCE_DEFENSIVE );
				ooo.PlanAction( CreateMovementAction( BasePos[i] ), true, true );
			}
		}
#ifdef DEVELOPER
		else
		{
			if ( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> ( EndAction() ) ReturnToBase je false, jednotky zustavaji teda tam kde jsou.";
				Print( str );
			}	
		}
#endif
	}
	
	//--------------------------------------------------------------------------
	// Healing Units
	//--------------------------------------------------------------------------
	void HealingUnits()
	{
		float tmpMinLive = 0.9;

		// Je jednotka opravena?
		if( GoHealer )
		{
			//Print( GoHealer.GetHealth() );
			WidgetMsg = GetName() + " Need Healer!"

			if( GoHealer.GetHealth() >= 0.92 )
			{
				// Print("Jednotka opravena!");
				GoHealer = NULL;
			}
		}

		// Vyhledam jednotku pro opravu a spocitam kolik je potreba celkove opravit
		for( int i = 0; i < UnitsCnt; i++ )
		{
			ObjectHandlerBase object = Units[i];

			Object objects[32];

			int objectCount = object.BuildObjectList( objects );

			if( objectCount == 0 ) continue;

			// Vybere si nejposkozenejsi jednotku
			for( int j = 0; j < objectCount; j++ )
			{
				Object o = objects[j];
				InteractiveEntity ent = o.GetEntity();
				float live = ent.GetHealth();

				if ( live < 0.9 )
				{
					if ( tmpMinLive > live )
					{
						tmpMinLive = live;

						// Pokud je nastaven priznak, jednotku pro opraveni nemenim
						if( GoHealer == NULL ) GoHealer = ent;
					}
				}
			}
		}
	}
	
	//--------------------------------------------------------------------------
	// HelpPrint
	//--------------------------------------------------------------------------
#ifdef DEVELOPER
	void HelpPrint()
	{
		if( TACTICS_DEBUG )
		{
			string str;
			Print( "[GUARD] >>> -----------------------------------------------------------------------" );
			Print( "[GUARD] >>> Help Print" );
			Print( EnemiesCnt );

			Print( "Enemy Defense:" );
			Print( EnemyMantaDefense );
			Print( EnemyWalrusDefense );

			Print( "Enemy Offense:");
			str = "Enemy AA Offense = "
			str += itoa( EnemyForce[ATTACK_AA] );
			Print( str );
			str = "Enemy AT Ofense = "
			str += itoa( EnemyForce[ATTACK_AT] );
			Print( str );
			str = "Enemy AL Ofense = "
			str += itoa( EnemyForce[ATTACK_AL] );
			Print(  str  );

			Print( "My Defense:");
			Print( MyMantaDefense );
			Print( MyWalrusDefense );

			Print( "My Offense:");
			str = "My AA Offense = "
			str += itoa( MyForce[ATTACK_AA] );
			Print( str );
			str = "My AT Ofense = "
			str += itoa( MyForce[ATTACK_AT] );
			Print( str );
			str = "My AL Ofense = "
			str += itoa( MyForce[ATTACK_AL] );
			Print(  str  );
			Print( "[GUARD] >>> -----------------------------------------------------------------------" );
		}
	}
#endif	
	//--------------------------------------------------------------------------
	//! Widget info
	//--------------------------------------------------------------------------
#ifdef DEVELOPER
	void TacticsWidgetInfo()
	{
		// handlery
		for( int i = 0; i < UnitsCnt; i++ )
		{
			ObjectHandlerBase myOhb = Units[i];
			WriteToHandlerWidget(myOhb, WidgetMsg, MSG_ERROR);
		}
	}
#endif	
	//--------------------------------------------------------------------------
	//! main update method
	//--------------------------------------------------------------------------
	void Update()
	{

		if ( g_Game.GetCommandCenter() == NULL )
		{
#ifdef DEVELOPER
			if( TACTICS_DEBUG )
			{
				string str = "[" + GetName() + "]" + " >>> UPDATE - neni CC - nic nedelej."
				Print( str );
			}
#endif
			return;
		}

		float newTime = g_Game.GetCampaignTime();
		float difTime = newTime - Time;
		float difTimeUpd = newTime - TimeUpd;

		
#ifdef DEVELOPER
		if ( TACTICS_DEBUG )
		{
			if( KeyState(KC_SCROLL) )
			{
				ClearKey(KC_SCROLL);
				debug
			}
		}
#endif
		if ( !Inizialized )
		{
#ifdef DEVELOPER
			if( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> Taktika neni inicializovana - inicializuj.";
				Print( str );
			}
#endif
			Inizialized = Init();
			return;
		}

#ifdef DEVELOPER
		if(DiagBool(AI_VISUAL_GLOBAL))
			TacticsWidgetInfo();
#endif
		if ( difTimeUpd > 60 )
		{
			TimeUpd = newTime;
			UpdateEnemies();
			UpdateMyUnits();
		}

		if ( UnitsCnt <= 0 )
			return;

		// Pokud existuje enemak
		if ( EnemiesCnt > 0 )
		{
			if ( difTime > 120 )
			{
#ifdef DEVELOPER
				if( TACTICS_DEBUG )
				{
					Print( "-------------------------- ---------------------------------------------------------------------------") ;
					str = "[" + GetName() + "]" + " >>> UPDATE - Baze vi o enemecich a prosel 5ti vterinovy timeout.";
					Print( str );
				}
				WidgetMsg = GetName() + " Have Enemies! " + itoa( EnemiesCnt );
#endif
				Time = newTime;
				Object o;
			
			    // prvni pokyn
				if ( !UnitsSended )
				{
#ifdef DEVELOPER
					if( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> UPDATE - Jednotky vyslany do akce";
						Print( str );
					}
#endif
					GoToAction();
					UnitsSended = true;
				}
			}
		}
		else
		{
			// Nejsou enemaci
			GoPosition = ZeroVec;
		
			if( difTime > 120 )
			{
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					str = "[" + GetName() + "]" + " >>> UPDATE - Nema Enemaky."
					Print( str );
				}

				WidgetMsg = GetName() + " No Enemies!"
#endif
		   	Time = newTime;
		   		
				// Navracim jednotky zpet ... toto je zduplikovane. Je mozne, ze uz to tu nemusi byt!
				if ( UnitsSended )
				{
#ifdef DEVELOPER
					if( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> Jednotky byly v akci stahnete se zpatky (mozna uz nemusi byt?).";
						Print( str );
					}
#endif
					EndAction();
					UnitsSended = false;
				}

				if ( TacticGotHealer )
					HealingUnits();
			}
		}
	}
	
	//--------------------------------------------------------------------------
	//! offer this tactic an object
	// should return true if tactic wants this object
	//--------------------------------------------------------------------------
	bool Offer( ObjectHandlerBase object )
	{
		//reinforcement nabizi... porovnam forces vs defend a dle toho prijimam
		// Print("nabidka");

		int reinWalrusDefense;
		int reinMantaDefense;
		int reinForce[3];
		
		// sila, jenz mi offer momentalne nabizi. Nastavim na nulu
		reinMantaDefense = 0;
		reinWalrusDefense = 0;
		reinForce[ATTACK_AA] = 0;
		reinForce[ATTACK_AT] = 0;
		reinForce[ATTACK_AL] = 0;

		Object objects[32];
		int objectCount = object.BuildObjectList( objects );

		if ( objectCount == 0 )
		{
#ifdef DEVELOPER
			if( TACTICS_DEBUG )
			{
				Print("--------------------------------------------------------------------------------------------------------------");
				string str = "[" + GetName() + "]" + " >>> OFFER - nic nenabizeji - nic neberu.";
				Print( str );
			}
#endif
			return false;
		}

		// Projedu pole jednotek. (muze byt grupa) a spocitam podporujici silu
		for( int i = 0; i < objectCount; i++ )
		{
			Object o = objects[i];
			InteractiveEntity ent = o.GetEntity();

			int type = ent.GetAttackType();

			if( type == ATTACK_NONE )
			{
#ifdef DEVELOPER
				if ( TACTICS_DEBUG )
				{
					str = "[" + GetName() + "]" + " >>> OFFER - jednotka bez attacku - to se nemelo stat.";
					Print( str );
				}
#endif
				continue;
			}
			else
				reinForce[type] = reinForce[type] + ent.GetAttackPower();

			type = o.GetType();
			switch( type )
			{
				case AI_MANTA :
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - jednotka je manta.";
						Print( str );
					}
#endif
					reinMantaDefense += ent.GetDefense();					
					break;
				
				case AI_WALRUS :
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - jednotka je walrus.";
						Print( str );
					}
#endif						
					if ( ent.IsHealer() == true )
					{
#ifdef DEVELOPER
						if ( TACTICS_DEBUG )
						{
							str = "[" + GetName() + "]" + " >>> OFFER - jednotka je healer.";
							Print( str );
						}
#endif
						reinWalrusDefense += 2;
						reinMantaDefense += 2;
					}

					reinWalrusDefense += ent.GetDefense();
					break;

				case AI_SOLDIER:
#ifdef DEVELOPER
					if ( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - jednotka je vojak ci droid - beru vzdy - neni to blbost???.";
						Print( str );
					}
#endif
					return true;
					break;
			}
		}
/*
		int enemyMantaForce = EnemyForce[ATTACK_AA];
		enemyMantaForce = enemyMantaForce + EnemyForce[ATTACK_AL];
		
		int enemyWalrusForce = EnemyForce[ATTACK_AT];
		enemyWalrusForce = enemyWalrusForce + EnemyForce[ATTACK_AL];

		int enemyForce = EnemyForce[ATTACK_AT];
		enemyForce = enemyForce + EnemyForce[ATTACK_AA];
		enemyForce = enemyForce + EnemyForce[ATTACK_AL];
		enemyForce *= 2;
		
		int allMyDefense = MyWalrusDefense + MyMantaDefense;
*/
		// Filtr
		
		// Ma AA utok... neposilam, pokud nemaji Manty
		if( reinForce[ATTACK_AA] > 0 && EnemyMantaDefense == 0 )
		{
#ifdef DEVELOPER
			if ( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> OFFER - NECHCI jednotku protoze ma AA silu a utocnik nema mantu.";
				Print( str );
			}
#endif
			return false;
		}

		// Ma AT zbrane... neposilam, pokud nemaji walrusy
		if( reinForce[ATTACK_AT] > 0 && EnemyWalrusDefense == 0 )
		{
#ifdef DEVELOPER
			if( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> OFFER - NECHCI jednotku protoze ma AT silu a utocnik nema walrusy.";
				Print( str );
			}
#endif
			return false;
		}

#ifdef DEVELOPER
		if( TACTICS_DEBUG )
		{
			str = "[" + GetName() + "]" + " >>> OFFER - jednotku BERU - protoze proste prosla podminkama.";
			Print( str );
		}
#endif
		return true;

/*
		// Rozhodujici system... taktika se snazi mit dvakrat vetsi defensi nez je utocna sila nepritele. Jak pro walrusy, tak pro manty
  		if( allMyDefense < enemyForce )
  		{
			if( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> OFFER - allMyDefense je mensi nez enemyForce.";
				Print( str );
			}

			// Manta AT posilam AA walrusy
			if ( EnemyForce[ATTACK_AT] > 0 && EnemyMantaDefense && reinWalrusDefense > 0 && reinForce[ATTACK_AA] > 0 )
			{
				if( TACTICS_DEBUG )
				{
					str = "[" + GetName() + "]" + " >>> OFFER - JEDNOTKU BERU - nepritel ma AT silu a ma manti obranu. Nabizena jednotka ma walrus obranu a navic AA silu."
					Print( str );
					Print("--------------------------------------------------------------------------------------------------------------");
				}
			    return true;
			}

			// Rozhodujici... radeji posilam walrusy, pokud je utok zalozen na AA utoku
			int eFAA = EnemyForce[ATTACK_AA] * 2;
			if ( eFAA > MyWalrusDefense )
			{
				if ( TACTICS_DEBUG )
				{
					str = "[" + GetName() + "]" + " >>> OFFER - nepritelova AAsila*2 je vetsi nez moje walrusi obrana";
					Print( str );
				}
				if( reinWalrusDefense > 0 )
				{
					if( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - JEDNOTKU BERU - protoze ma walrusi obranu" 
						Print( str );
						Print("--------------------------------------------------------------------------------------------------------------");
					}
					return true;
				}
				else
				{
					if( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - JEDNOTKU NEBERU - protoze nema walrusi obranu";
						Print( str );
						Print("--------------------------------------------------------------------------------------------------------------");
					}
					return false;
				}
			}

			// radeji posilam manty, pokud je utok zalozen na AT utoku
			int eFAT = EnemyForce[ATTACK_AT] * 2;
			if( eFAT > MyMantaDefense )
			{
				if ( TACTICS_DEBUG )
				{
					str = "[" + GetName() + "]" + " >>> OFFER - nepritelova ATsila*2 je vetsi nez moje manti obrana.";
					Print( str );
				}
				if ( reinMantaDefense > 0 )
				{
					if( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - jednotku BERU - nabizena jednotka ma mantaDefense.";
						Print( str );
						Print("--------------------------------------------------------------------------------------------------------------");
					}
					return true;
				}
				else
				{
					if( TACTICS_DEBUG )
					{
						str = "[" + GetName() + "]" + " >>> OFFER - jednotku NEBERU - nabizena jednotka nema mantaDefense.";
						Print( str );
						Print("--------------------------------------------------------------------------------------------------------------");
					}
					return false;
				}
			}

			if( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> OFFER - jednotku BERU - protoze proste neni lepsi.";
				Print( str );
				Print("--------------------------------------------------------------------------------------------------------------");
			}
			return true;
		}

		if( TACTICS_DEBUG )
		{
			str = "[" + GetName() + "]" + " >>> OFFER - jednotku neberu protoze allMyDefense je vetsi nebo rovno enemyForce.";
			Print( str );
			Print("--------------------------------------------------------------------------------------------------------------");
		}
		return false;
*/
	}

	//! ask this tactic if it still needs an object
	// if this returns false, the object in question may be automatically removed
	// by tactic manager
	bool Needs( ObjectHandlerBase object )
	{
		// keep everything on attack
		if ( EnemiesCnt > 0 )
		{
#ifdef DEVELOPER
			if ( TACTICS_DEBUG )
			{
				string str = "[" + GetName() + "]" + " >>> NEED - mam nepritele, jednotky potrebuju"
				Print( str );
			}
#endif
			return true;
		}

#ifdef DEVELOPER
		if( TACTICS_DEBUG )
		{
			str = "[" + GetName() + "]" + " >>> NEED - nemam nepritele, vem si je"
			Print( str );
		}
#endif
		return false;
	}

	//! validate tactic
	// if false is returned, this instance will most likely be deleted
	bool Validate()
	{
		return true;
	}

	//! should return estimate of needed units of given type
	int EstimateUnitNeed( int type )
	{
#ifdef DEVELOPER
		if( TACTICS_DEBUG )
			Print( "[GUARD] >>> EstimateUnitNeed - leze to sem nekdy?" );
#endif
		return 0;
	}

	//! get need score of this component
	int NeedScore()
 	{
		int enemyForce = EnemyForce[ATTACK_AL] + EnemyForce[ATTACK_AA] + EnemyForce[ATTACK_AT];
		int allEnemyDefense = EnemyWalrusDefense + EnemyMantaDefense;

		int myForce = MyForce[ATTACK_AL] + MyForce[ATTACK_AA] + MyForce[ATTACK_AT];
		int allMyDefense = MyWalrusDefense + MyMantaDefense;
		
		if ( allMyDefense < enemyForce || myForce < allEnemyDefense )
		{
#ifdef DEVELOPER
			if ( TACTICS_DEBUG )
			{
				string str = "[" + GetName() + "]" + " >>> NeedScore - nemam dost sily vrat prioritu: " + itoa( GetBasePriority());
				Print( str );
				Print( UnitsCnt );
				Print( EnemiesCnt );
			}
#endif
			return GetBasePriority();
		}
		else
		{
#ifdef DEVELOPER
			if ( TACTICS_DEBUG )
			{
				str = "[" + GetName() + "]" + " >>> NeedScore - mam dost sily, vracej prioritu -1";
				Print( str );
				Print( UnitsCnt );
				Print( EnemiesCnt );
			}
#endif
			return -1;
		}
	}
};
/**@}*/