/** \defgroup StrategicAPI Strategic API */
/** \defgroup MissionsAPI Missions API */
/** \defgroup VehiclesAPI Vehicles API */
/** \defgroup GameAPI Game API */
/** \defgroup AIAPI AI internal API */

class World;
class CCPlayer;
class RepairSystem;
class InteractiveRepairZone;
class EntityBasic;
class InteractiveEntity;
class UniPost;
class Manta;
class Building;
class Activator;
class Item;
class Unit;
class Door2;
class Walrus;
class Carrier;
class Character;

Unit didrone;
Unit dicurrent;
Unit dicurrentprev;
Unit dicurrentprev2;
Unit dicurrentprev3;
Unit diprevious;
Unit diunit[9];
Unit diunitprev[9];
Unit diunit1;
Unit diunit2;
CCPlayer diPlayerUEC;
CCPlayer diPlayerAPA;
Walrus ditempwalrus;
Manta ditempmanta;
Carrier dicarruec;
Carrier dicarrapa;
Character didroid[33];
InteractiveEntity didroidleader[17];
InteractiveEntity dipier[5];
InteractiveEntity dipump[7];
InteractiveEntity diresearch;
InteractiveEntity dicc;
InteractiveEntity dipresetunit;
InteractiveEntity dicarrscanresult[100];
InteractiveEntity ditempentity;
InteractiveEntity ditaggedentity;
InteractiveEntity dibarque;
InteractiveEntity diddrone[5];
InteractiveEntity dispawnstation[20];
InteractiveEntity diemptyunits[100];
InteractiveEntity didebris[150];
InteractiveEntity dipointedentity;
InteractiveEntity ditacpointedentity;
InteractiveEntity dimantadummy[5];
InteractiveEntity dineutturret[150];
InteractiveEntity dihammerhead;
InteractiveEntity dicontainer;
InteractiveEntity dipipunit;
InteractiveEntity direnderentity;
InteractiveEntity direnderentityprev;
InteractiveEntity dilastentityclicked[9];
InteractiveEntity diunitcomponent[90];
InteractiveEntity disatelliteentity[200];
Unit diattackmanta[10];
Door2 dibarquedoor;
Door2 didronedoor;
Door2 didronesdoor;

/** \addtogroup MissionsAPI */
/**@{*/
//! ancestor for almost all entities in the game
class GameEntity extends _entity
{
	//! Retrieve position
	proto native vector GetPosition();
	//! Set position
	proto native void SetPosition( vector vPos );

	//! Retrieve orientation
	proto native vector GetOrientation();
	//! Set orientation
	proto native void SetOrientation( vector vOrientation );

	//! Retrieve horizontal orientation only
	proto native float GetHOrientation();

	//! Calculate local position of other entity to this entity
	proto native vector GetLocalPos( vector vPos );
	//! Calculate global position to this entity from local position
	proto native vector GetGlobalPos( vector vPos );

	//! Get entity name
	proto native owned string GetName();

	//! Assigned player (if any)
	proto native CCPlayer GetPlayer();

	//! Set assigned player and remap immediately
	proto native void ForcePlayer( CCPlayer pPlayer );
	//! Set assigned player
	proto native void SetPlayer( CCPlayer pPlayer );

	//! get remaining health (0.0 - 1.0)
	proto native float GetHealth();
	//! Get real unit health
	proto native float GetAbsHealth();

	//! get remaining fuel (if any)
	proto native float GetFuel();

	//! Damage (relative 0.0 .. 1.0)
	proto native bool Damage( float fDamage );
	//! Heal (relative 0.0 .. 1.0)
	proto native void Heal( float fHealth );

	//! enable/disable physics interactions
	proto native void EnablePhysics( bool bEnable );
	
	//! Show with all sub elements
	proto native void EnableShow();
	//! Hide with all sub elements
	proto native void DisableShow();
	
	void Show()
	{
		EnableShow();
	}
	
	void Hide()
	{
		DisableShow();
	}
	
	//! Disable usability of this entity
	proto native void DisableUse();
	//! Enable usability of this entity
	proto native void EnableUse();
	//! Check if use enabled
	proto native bool IsUseEnabled();
	//! Set usability by enemy On or Off
	proto native void SetUsableByEnemy( bool bUsable ); 
	//! Check if usable by enemy
	proto native bool IsUsableByEnemy();

	//! Start activation process
	proto native void StartActivation( float fLength ); 
	//! Cancel activation and set new progress (0.0 .. 1.0)
	proto native void CancelActivation( float fProgress );
	//! Get current progress (0.0 .. 1.0), if >= 1.0, progress stays on same value
	proto native float GetActivationProgress();
	//! Set activation time (sec)
	proto native void SetActivationTime( float fTime );
	
	
	//! Play dialog (localized text + sound)
	proto void Say( string id, float overtime );
	
	GameEntity Test(int toKoukasVole)
	{
	}
	
	//! Called when entity loads self
	event void OnLoad(World _world)
	{
	}

	//! Can be called and used like the EOnTick event
	event void OnLocalTick()
	{
	}
	//! Called whenever player tries to use this item
	event void OnUse( CCPlayer pPlayer )
	{
	}

	//! get unit position on strategic display
	proto native vector GetStrategyPos();


	//! Play game sound
	proto native void PlayGameSound( string sSoundShader );

	//! Show and enable entity function
	proto native void EnableComplex();
	//! Hide and disable entity function
	proto native void DisableComplex();
	//! Check if disabled complexly
	proto native bool IsComplexlyDisabled();

	//! Position shape mark into world for debug purposes
	proto native void DebugMark();
	//! Position shape mark into map
	proto native void DebugMapMark( float fX, float fY );

	//! Destroy entity
	proto native void Destroy();

	//! Apply physics impulse on entity
	proto native void ApplyImpulse( vector vForce );

	//! Set entity current invincibility
	proto native void SetInvincibility( bool bEnable );
	//! Get entity current invincibility
	proto native bool GetInvincibility();

	//! Check if unit is currently cargo
	proto native bool IsCargo();
	//! Get cargo transfer
	proto native Manta GetCargoTransfer();

	//! Get current mission time
	proto native float GetMissionTime( float fDivider );

	//! Get distance to another entity
	proto native float GetDistanceTo( GameEntity pEntity );

	//! On various game event
	event void OnGameEvent( string sEventName )
	{
	}

};

/**@}*/

/** \addtogroup MissionsAPI */
/**@{*/
//! Scripted timer
class ScriptedTimer extends GameEntity
{
	int Timeout;
	int default_timeout;
	bool timerActive;
	
	void Start()
	{
		timerActive = true;
		default_timeout = Timeout;
		SetTick( 1 );
	}
	
	void Stop()
	{
		timerActive = false;
		Timeout = default_timeout;
		RemoveTick();
		g_Script.Call(this, "OnTime", 0);
	}
	
	void Interrupt()
	{
		timerActive = false;
		Timeout = default_timeout;
		RemoveTick();
	}
	
	void EOnTick(_entity other, int extra)
	{
		if (timerActive) {
			Timeout--;
			if( Timeout == -1 )
				Stop();
		} else {
			RemoveTick();
			string printStr = "ScriptedTimer: Calling tick function despite timer being offline! Removing tick! Timeout = " + itoa(Timeout);
			Print(printStr);
		}
	}
	
	event void OnTime()
	{
	}
};
/**@}*/

/** \addtogroup MissionsAPI */
/**@{*/
//! Scripted FSM
class ScriptedFSM extends GameEntity
{
	string State;
	string states
	float Period;
	
	float GetPeriod()
	{
		return Period;
	}

	bool ValidateState(string state)
	{
		string lstates = states;
		while(true)
		{
			string token;
			int r = ParseStringEx(lstates, token);
			if(r == 0)
				break;
			if(r == 1)
				continue;
			
			if(token == state)
				return true; 
		}
		
		DPrint(String(ClassName(this) + " detected unknown state " + state));
		return false;	
	}

	bool IsState(string state)
	{
		if(!ValidateState(state))
		    return false;
		    
		if(state == State)
			return true;
		
		return false;
	}
	void SetPeriod(float timeSlice)
	{
		Period = timeSlice;

		if(timeSlice)
			SetTick(timeSlice);
		else
			RemoveTick();
	}

	event void OnChange(local string state, local string oldState)
	{
	}

	event void OnState(local string state)
	{
	}

	void EOnTick(_entity other, int extra)
	{
		string funcName = "OnState_" + State;
		if(!g_Script.Call(this, funcName, 0))
			OnState(State)		
	}
	
	void SetState(local string newState)
	{
		if(!ValidateState(newState))
		    return;

		if(newState != State)
		{
			DPrint(String(ClassName(this) + ": " + newState));

			local string oldState = State;
			State = newState;
			
			string funcName = "OnChange_" + newState;
			g_Script.Call(this, funcName, oldState);
			OnChange(newState, oldState)
/*			
			int ev = GetFlags(this) & EV_TICK
			if(ev != 0 && State == newState)
				EOnTick(this, 0);*/
		}	
	}

	void Start(string initialState, float period)
	{
		Period = period;

		if(period)
			SetTick(period)
		SetState(initialState)
	}
};
/**@}*/

/** \addtogroup MissionsAPI */
/**@{*/
//! Common ancestor for scripted entity
class ScriptedEntity extends GameEntity
{

	// then content becomes enabled
	event void OnSubsidiaresEnabled()
	{
	}
	// then content becomes disabled
	event void OnSubsidiaresDisabled()
	{
	}

	// control sequence
	event void OnFrame( InteractiveEntity pActor, float fTimeSlice )
	{
//		Print("OnFrame:: ScriptedEntity");
	}

	//! On carrier unit game event
	event void OnCarrierUnitEvent( InteractiveEntity pUnit, string sEventName )
	{
	}
	//! On carrier unit+other game event
	event void OnCarrierOtherEvent( InteractiveEntity pUnit, InteractiveEntity pOther, string sEventName )
	{
	}

	//! Enable children parented in map
	proto native void EnableSubsidiaries();
	//! Disable children parented in map
	proto native void DisableSubsidiaries();
	//! Check if children enabled
	proto native bool AreSubsidiariesEnabled();

};
/**@}*/
class SeaBottom extends GameEntity
{
};

class WindEmitor extends GameEntity
{
};

//! SpriteRefract
class SpriteRefract extends GameEntity
{
//	cache 16

	reference float RefrScale;
	reference float Chromacity;
	proto native void SetSize(float sx, float sy);
	proto native void SetRefraction(float val);
};

class SpriteRefractColor extends SpriteRefract
{
	reference int Color;
};

class HitDistort extends SpriteRefract
{
};

class Decal extends GameEntity
{
};

class Light extends GameEntity
{
	reference float Color[4];
};

class WorldLight extends GameEntity
{
};

class StaticLight extends Light
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! can be inserted into the world via WB
class DynamicLight extends Light
{
	proto native void SetColor(vector rgb);
	proto native void SetRadius(float radius);
	proto native void SetIntensity(float intensity);
};
/**@}*/
class NullObject extends GameEntity
{
};


class DummyPoint extends NullObject
{
};
/*
class DummyArea extends DummyPoint
{
//	vector mins, maxs;
	float radius, height;
	
	void Init()
	{
		vector mins, maxs;
		mins[0] = -radius;
		mins[1] = -radius;
		mins[2] = height * -0.5;
		maxs[0] = radius;
		maxs[1] = radius;
		maxs[2] = height * 0.5;
		CreateModel(this, mins, maxs);
		SetFlags(this, TFL_SOLID);
		UpdateEntity(this);
	}
		
	void DummyArea()
	{
		Init();
	}

	void Cleanup()
	{
		RemoveModel(this);
	}
		
	void ~DummyArea()
	{
		Cleanup();
	}
};
*/

/** \addtogroup MissionsAPI */
/**@{*/
//! can be inserted into map in WB with some default params
class SpawnPoint extends NullObject
{
	//! Spawn default (what is set from map)
	proto native InteractiveEntity SpawnDefault( CCPlayer pPlayer );
	//! Spawn custom unit
	proto native InteractiveEntity SpawnUnit( CCPlayer pPlayer, int eStyle );
	//! Spawn custom unit at custom position
	proto native InteractiveEntity SpawnUnitAt( CCPlayer pPlayer, int eStyle, vector vecTranslation );
	
	//! Get default style
	proto native int GetDefaultStyle();
};
/**@}*/

class WayPointBase extends NullObject
{
};

class WayPoint extends WayPointBase
{
};

/** \addtogroup StrategicAPI */
/**@{*/
//! IslandKeyPoint ancestor
class KeyPoint extends WayPointBase
{
	proto void GetNeighbors( KeyPoint neighs[], int cnt );
};
/**@}*/

//! RoutePointBase
class RoutePointBase extends WayPointBase
{
	//! agent arrived at this waypoint
	event void OnAgentArrived( Unit agent )
	{
	}
};

class RoutePoint extends RoutePointBase
{
};

class Route extends RoutePointBase
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! is used for strategic AI navigation on island when fighting on non-loaded island
class Trigger extends WayPointBase
{
	// control
	proto native void Activate();
	proto native void Deactivate();
	proto native bool IsActive();

	// all inside units
	//DO NOT CALL FROM OnEntered event - result will be doubtful
	proto void  GetUnitsInside( InteractiveEntity units[], int unitCnt );

	//! test position against trigger
	proto native bool Inside( vector pos );

	/*!	\name Events	*/
	//@{

	//! activated by entity
	event void OnEntered( InteractiveEntity entity )
	{
	}
	//! entity left
	event void OnLeft( InteractiveEntity entity )
	{
	}
	//! activated by first entity
	event void OnFirstEntered()
	{
	}
	//! last entity left
	event void OnLastLeft()
	{
	}
	//! function that can be called from within the game
	event void OnSeizedBy()
	{
	}

	//@}
};
/**@}*/

class IslandAI extends WayPointBase
{


};



//! some pre-defined group formations (SEE formations.xml)
const int FORMATION_CONVOY = 0;
const int FORMATION_WING = 1;
const int FORMATION_SQUARE = 2;
const int FORMATION_LINE = 3;
const int FORMATION_U = 4;
const int FORMATION_MANTA_WING = 10;

/** \addtogroup MissionsAPI */
/**@{*/
//! is used to group more units and give them orders through this entity (leader)
class GroupEntity extends WayPointBase
{
	//! add member unit to the squad
	//NB: you got to call Build() after you've added all members!
	proto native void AddMember( Unit member );

	//! build the squad after all members have been added
	proto native bool BuildSquad( int preferredFormation );

	//! change formation
	// returns false if formation could not be formed from provided units
	proto native bool SetFormation( int id );

	//! disband the group (irreversible unless manually built again)
	proto native void Disband();

	//! temporaly disband the group so it can be easily regrouped later
	proto native void TempDisband();

	//! regroup after TempDisband() has been called
	proto native void Regroup();

	proto native void EnableAI();
	proto native void DisableAI();

	//! set unit stance (see cc.h for defines)
	proto native void SetStance( int stanceID );

	//! behavior control
	proto native void EnableBehavior( string name );
	proto native void ClearBehavior(); // clear any set behavior

	//! retrieve number of members alive
	proto native int GetNumMembers();

	//! go to position
	proto native void GoTo( vector target, int acton );

	//! stop & wait @ position
	proto native void Wait( float timeInSeconds, int action );

	//! OBSOLETE!!! follow route
	proto native void FollowRoute( string name, bool slowly, int action );

	//! follow route
	proto native void Follow( Route r, int action );

	//! attack target
	proto native void Attack( InteractiveEntity target, int action );

	//! assist ally
	proto native void Assist( InteractiveEntity target, int action );

	//! help ally
	proto native void Help( InteractiveEntity target, int action );

	//! clear unit plan
	proto native void ClearPlan();

	//! check for any planned actions
	proto native bool PlanEmpty();


	//! teleport to destinations
	//NB: the 'angle' parameter does not work for groups
	proto native void Teleport( vector at, float angle );

	//! destroyed
	event void OnDestroyed()
	{
	}

	//! spotted an enemy
	event void OnEnemySpotted( InteractiveEntity enemy )
	{
	}

	//! attacked by an enemy for the first time
	event void OnAttacked( InteractiveEntity attacker )
	{
	}

	//! completed an action
	event void OnActionDone( class action )
	{
	}

	//! entering idle state
	event void OnPlanEmpty()
	{
	}	

};
/**@}*/
class SoundAncestor extends NullObject
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! can inserted into the world via WB with some default params
class SoundPoint extends SoundAncestor
{
	//! Start or Stop sound periodic
	proto native void EnableSoundPeriodic( bool bEnable );
	//! Start or Stop sound loop
	proto native void EnableSoundLoop( bool bEnable );
	
	//! Check if loop is playing
	proto native bool IsPlayingLoop();
		
};
/**@}*/

class SoundAmbient extends SoundAncestor
{
};

class DockPoint extends NullObject
{
};

//! possible unit attack types
const int ATTACK_NONE = -1;
const int ATTACK_AA = 0;
const int ATTACK_AT = 1;
const int ATTACK_AL = 2;

/** \addtogroup MissionsAPI */
/**@{*/
//! ancestor for all Interactive entities in game (entities with AI)
class InteractiveEntity extends GameEntity
{
	reference int LightColor;

	proto native class GetAI();
	proto native void EnableAI();
	proto native void DisableAI();

	//! Suspend On AI
	proto native void PlanSuspend();
	//! Suspend Off AI
	proto native void PlanEnable();
	//! Clear plan
	proto native void PlanClear();
	//! Check if has some plan at all
	proto native bool HasPlan();
	//! Check if plan suspended
	proto native bool IsPlanSuspended();

	//! Check if unit is docked
	proto native bool IsDocked();
	//! check if unit is docking right now
	proto native bool IsDocking();

	//! Dock
	proto native void ProcessDock();
	//! Undock
	proto native void ProcessUndock();
	//! Enable or disable AutoPilot
	proto native void AutoPilot( bool bEnable );

	//! Get current cargo
	proto native GameEntity GetCargo();

	// combat props
	proto native void SetAttacking( bool set );
	proto native bool IsAttacking();
	proto native void SetAttackable( bool set );
	proto native bool IsAttackable();

	//! set active stance
	proto native void SetStance( int stanceID );
	//! get it back
	proto native int GetStance();

	//! behavior control
	proto native void EnableBehavior( string name );
	proto native void ClearBehavior(); // clear any set behavior

	//! Set captive mode for entity
	proto native void SetCaptive( bool set );
	//! Check if captive mode set
	proto native bool GetCaptive();

	//! Cal nearby allies for help
	// 'radius' - the search diameter allies will be searched for. if <= 0, AI known allies will be
	// called (preferred option - both performance and functionality wise)
	proto native int CallForHelp( float radius );

	//! Get index from carrier, if unit comes from it (otherwise -1)
	proto native int GetCarrierIndex();

	//! Check if this unit is from carrier
	proto native bool IsCarrierUnit();
	//! Check if this unit is in drive mode
	proto native bool IsDriveMode();

	//! Check id this is a healing unit
	proto native bool IsHealer();

	//! Equip all items o styles
	proto native void EquipStyle( int eStyle );
	//! Equip specific item
	proto native void AddEquipment( int eItem );
	//! Clear equipment
	proto native void ClearEquipment( int eItem );

	//! Check item equipped
	proto native bool HasEquip( int eDBEnum );

	//! Manually use equipment
	proto native void UseItem( int eDBEnum );

	//! Change entity style in runtime
	proto native void ChangeEntityStyle( int eStyle );

	//! Enable equipment functionality
	proto native void EnableItems();

	//! Selects entity as controlled, for active player
	proto native void SelectForPlayer();
	//! Check if entity is currently controlled by it's player
	proto native bool IsSelected();
	//! Selects entity as players actor
	proto native void SelectAsPlayerActor();
	//! Check if entity is selected as wingman
	proto native bool IsWingman();
	//! Check if entity is current players actor
	proto native bool IsActor();
	//! Load actor into this entity
	proto native void LoadActor();
	//! Drop actor out of this entity
	proto native void DropActor();
	//! Drop actor out of this entity
	proto native bool DropOtherActor( InteractiveEntity pEntity, float fRange, float fTurn, int iSegments );
	//! check 
	proto native bool IsActorOnBoard();

	//! On use event
	proto native void OnExternUse( GameEntity pEntity );

	//! Invoke camera shake from script
	proto native void OnCustomCameraShake( int eShake, float fIntensity );

	//! Will set proper message for current action
	proto native void SetActivationMessage( string sMessage );

	//! Enable material remap on SetPlayer() 
	proto native bool EnableMaterialRemap();

	//! Get repair system handle
	proto native RepairSystem GetRepairSystem();

	//! Get particular assembly
	proto native EntityBasic GetAssembly();
	
	//! Return current ammo count
	proto native int GetAmmo();
	//! Add ammo to entity's current weapon
	proto native void AddAmmo( int iAmmo );

	//! Set fuel to particular count in range 0.0 .. 1.0
	proto native void SetFuel( float fFuel );
	

	//! Teleport unit to position, using defined horizontal angle
	proto native void Teleport( vector vCoords, float fHAngle );
	//! Teleport to entity (DummyPoint or something) including full orientation
	proto native void TeleportToEntity( GameEntity pEntity );

	//! Attack type of entity (see above class declaration)
	proto native int GetAttackType();
	//! Attack power of entity (retrieve exact type using method above)
	proto native int GetAttackPower();
	//! Defense power of entity
	proto native int GetDefense();
	//! Support utility of entity
	proto native int GetSupportUtility();

	//! Select higher angle for ballistic shooting (disabled by default)
	proto native void EnableHighBallistic( bool enable );
	proto native bool HighBallisticEnabled();

	//! discovery queries
	proto native bool WasDiscovered();
	proto native bool VisibleOnMap();
	proto native bool VisibleOnRadar();
	proto native bool VisibleOnHUD();

	//! Check if camera as set to exterior
	proto native bool IsCameraExterior();
	//! Enable or disable exterior camera
	proto native void EnableCameraExterior( bool bExterior );

	//! Shield health
	proto native void SetShieldHealth( float fHealth );
	//! Shield shutdown time
	proto native void SetShieldShutdownTime( float fTime );
	//! Shield recharge time
	proto native void SetShieldRechargeTime( float fTime );
	//! Shield recharge count
	proto native void SetShieldRechargeCount( float fCount );
	//! Next shield hurt event idle (next event will be called after this idle)
	proto native void SetShieldEventIdle( float fIdle );

	//! Get unit skill
	proto native float GetSkill();
	//! Set unit skill
	proto native void SetSkill( float fSkill );

	//! Determine if can be target for repair ray
	proto native  bool IsRepairRayReceiever();

	//! Travel indoor
	proto native void TravelIndoor( float fLerp );

	//! Get current item
	proto native Item GetCurrentItem();

	//! Start fire
	proto native void ForceStartFire();
	//! Stop fire
	proto native void ForceStopFire();

	//! Check if entity is indoor
	proto native bool IsIndoor();

	//! Check if having assist action on plan
	proto native bool HasAssistAction();

	//! Get activator if exist (otherwise return NULL!)
	proto native Activator GetActivator();

	//! Sent to hack of target building (find path+hack)
	proto native bool InvokeHackOfBuilding( Building pBuilding );

	//! Deplete capsule if equipped
	proto native void DepleteCapsule();

	/*!	\name Events	*/
	//@{

	//! destroyed entity (explosion or something)
	event void OnDestroyed()
	{
	if ( dicarrscan > 1 && di_tactical_scan_limit && this.GetPlayer() != diPlayerUEC )
		{
		for ( int didestcount = 0; didestcount < 100 && dicarrscanresult[didestcount] != NULL; didestcount++ )
			{
			if ( this == dicarrscanresult[didestcount] )
				FindAnyWidget ( ditacticalrootwidget[didestcount], "Healthlevel" ).SetSize ( 0, 0 );
			}
		}
	for ( didestcount = 0; didestcount < 10; didestcount++ )
		{
		if ( this == diattackmanta[didestcount] )
			{
			disearchpoint[didestcount] = this.GetPosition();
			diattackmanta[didestcount] = NULL;
			dicarrattacktimer = 0;
			}
		}
	}

	//! deleted from map
	event void OnDeleted()
	{
	for ( int didestcount = 0; didestcount < 200; didestcount++ )
		{
		if ( this == disatelliteentity[didestcount] )
			disatelliteentity[didestcount] = NULL;
		}
	for ( didestcount = 0; didestcount < 10; didestcount++ )
		{
		if ( this == diattackmanta[didestcount] )
			diattackmanta[didestcount] = NULL;
		}
	for ( didestcount = 0; didestcount < 150; didestcount++ )
		{
		if ( this == didebris[didestcount] )
			didebris[didestcount] = NULL;
		}
	if ( this == diprevious )
		diprevious = NULL;
	}

	//! spotted an enemy
	event void OnEnemySpotted( InteractiveEntity enemy )
	{
	}

	//! attacked by an enemy for the first time
	event void OnAttacked( InteractiveEntity attacker )
	{
	}

	//! hit by any damage
	event void OnHit( InteractiveEntity attacker, float damage )
	{
	if ( this.GetPlayer() == diPlayerAPA && attacker.GetPlayer() == diPlayerAPA )
		this.Heal( damage );
	if ( this.GetPlayer() != diPlayerUEC )
		{
		if ( dicarrscan > 1 && di_tactical_scan_limit )
			{
			for ( int didestcount = 0; didestcount < discanmaxdisplay; didestcount++ )
				{
				if ( this == dicarrscanresult[didestcount] )
					{
					float dihealthfloat = this.GetHealth() * 100 * 1.35;
					int dihealthint = dihealthfloat;
					FindWidget ( ditacticalrootwidget[didestcount], "Healthlevel" ).SetSize ( dihealthint, 10 );
					break;
					}
				}
			}
		if ( attacker == dicurrent && !dicarrierselectwidget )
			{
			for ( didestcount = 0; didestcount < 200; didestcount++ )
				{
				if ( this == disatelliteentity[didestcount] )
					{
					ditargethittimer = diframerate / 2;
					break;
					}
				}
			}
		}
	}

	//! completed an action
	event void OnActionDone( class action )
	{
	}

	//! entering idle state
	event void OnPlanEmpty()
	{
	}

	//! On RepairSystem zone destroyed
	event void OnRepairZoneDestroy( int iZone )
	{
	}
	//! On RepairSystem zone repair
	event void OnRepairZoneRepair( int iZone )
	{
	}

	//! Called whenever player tries to use this item
	event void OnShieldDown( int iRechargesRemain )
	{
	}
	//! Called whenever player tries to use this item
	event void OnShieldRegenerated()
	{
	}
	//! Called whenever player tries to use this item
	event void OnShieldHurt( float fHealthBefore, float fHealthAfter )
	{
	}

	//! On activation start
	event void OnActivationStart()
	{
	}
	//! On activation finished
	event void OnActivationFinished()
	{
	}
	//! On activation break
	event void OnActivationBreak()
	{
	}

	//! Called when actor enters this entity
	event void OnActorLoad( InteractiveEntity pActor )
	{
	}
	//! Called when actor leaves this entity
	event void OnActorDrop( InteractiveEntity pActor )
	{
	}	
	
	//! On various game event
	event void OnGameEvent( string sEventName )
	{
		switch(sEventName)
		{
		case "used":
			PlayGameSound("fx/activator_used");
			break;
		case "usestart":
			PlayGameSound("fx/activator_usestart");
			break;
		case "usefinished":
			PlayGameSound("fx/activator_usefinished");
			break;
		case "usefail":
			PlayGameSound("fx/activator_usefail");
			break;
		case "usebreak":
			PlayGameSound("fx/activator_usefail");
			break;
		}
	}

	//@}
};
/**@}*/

class NonInteractiveEntity extends GameEntity
{
};

class SoundEnvironment extends NonInteractiveEntity
{
};

class StrategyGrid extends NonInteractiveEntity
{
};

//! Colorizable
class Colorizable extends NonInteractiveEntity
{
	reference int cColor;
	reference int eColor;
	reference float uvRotate[2];
	reference float GeomScale;
};

class StrategyPath extends Colorizable
{
};

class Exquisite extends Colorizable
{
};

class StrategyTelemetry extends Colorizable
{
};
class StrategyRadarLine extends StrategyTelemetry
{
};
class StrategyLink extends StrategyTelemetry
{
};
class StrategyArc extends StrategyTelemetry
{
};
class CarrierTelemetry extends StrategyTelemetry
{
};
class IslandTelemetry extends StrategyTelemetry
{
};
class UnitTelemetry extends StrategyTelemetry
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! ancestor for component type entity like Activator, Door2, ProjectileCaster
class CPart extends NonInteractiveEntity
{
	//! enable/disable associated obstacle (if any)
	proto native void EnableAIObstacle( bool enable );

	//! specific Show/Hide methods
	void Show()
	{
		SetFlags(this, TFL_VISIBLE|TFL_SOLID);
		EnablePhysics( true );
		EnableAIObstacle( true );
	}
	void Hide()
	{
		ClearFlags(this, TFL_VISIBLE|TFL_SOLID);
		EnablePhysics( false );
		EnableAIObstacle( false );
	}

	//! On component destroyed
	event void OnDestroy()
	{
	}

	//! On component contact
	event void OnContact()
	{
	}

	//! Enable cammo remap
	proto native void EnableCammoRemap();
	//! Disable cammo remap
	proto native void DisableCammoRemap();
	//! Check if cammo is currently used
	proto native bool IsCammoRemap();
	//! Enable or disable contact event
	proto native void EnableContactEvent( bool bEnable );

};
/**@}*/

//! Component
class Component extends CPart
{
	reference int LightColor;
};

/** \addtogroup MissionsAPI */
/**@{*/
//! entity which player can use
class Activator extends CPart
{

	//! Various game events (used, finished etc)
	event void OnGameEvent( string sEventName )
	{
		switch(sEventName)
		{
		case "used":
			PlayGameSound("fx/activator_used");
			break;
		case "usestart":
			PlayGameSound("fx/activator_usestart");
			break;
		case "usefinished":
			PlayGameSound("fx/activator_usefinished");
			break;
		case "usefail":
			PlayGameSound("fx/activator_usefail");
			break;
		case "usebreak":
			PlayGameSound("fx/activator_usefail");
			break;
		}
	}

	//! Called whenever player tries to use activator
	event void OnUse( CCPlayer pPlayer )
	{
	}
	//! Called when activation starts
	event void OnActivationStart()
	{
	}
	//! Called when activation was finished (successfuly)
	event void OnActivationFinished()
	{
	}
	//! Called when activation was interupted
	event void OnActivationBreak()
	{
	}
	//! Called when activation fails
	event void OnActivationFail()
	{
	}

	//! Checks if activation is possible
	event int IsActivationPossible( GameEntity pEntity )
	{
		return 1;
	}

};
/* @}*/

class Hangar extends CPart
{

	//! Enter deploy preparation state
//	proto native void Deploy( int iSlot, int eEntityStyle );
	//! Enter dock preparation state
//	proto native void Dock( int iSlot, InteractiveEntity pEntity );
	//! Check if entity can deploy right now
//	proto native int IsEnabled( int iSlot );
	//! Check entity can dock
//	proto native int IsWorking( int iSlot );

};
/** \addtogroup MissionsAPI */
/**@{*/
//! API for animated objects (inserted via WB)
class Animated extends CPart
{
	//! Rotate smoothly to specified angle in time
	proto native void RotateToHAngle( float fAngle, float fTime );

	//! Is Rotation
	proto native int IsRotating();


	//! Set animation speed in seconds
	proto native void SetAnimSpeed( float fSpeed );
	//! Set animation progress in interval 0.0 .. 1.0
	proto native void SetAnimProgress( float fProgress );

	//! Start forward animation from current progress
	proto native void StartAnimForward();
	//! Start backward animation from current progress
	proto native void StartAnimBackward();
	//! Stop animation
	proto native void StopAnim();
	//! Enable animation loop
	proto native void EnableAnimLoop( bool bEnable );

	//! Animation finished (current progress returned)
	event void OnAnimEnd( float fProgress )
	{
	}

};

//! Door's API
class Door2 extends CPart
{
	//! Process door to open
	proto native void Open();
	//! Process door to close
	proto native void Close();

	//! Process door open after specified period
	proto native void AutoOpen( float fTime );
	//! Process door close after specified period
	proto native void AutoClose( float fTime );

	//! Check if opened
	proto native bool IsOpened();
	//! Checks if doors is closed
	proto native bool IsClosed();
	//! Checks if doors is locked
	proto native bool IsLocked();
	//! Process door to lock
	proto native void Lock();
	//! Process door to Unlock
	proto native void Unlock();

	//! Called when opened
	event void OnOpen()
	{
	if ( this == dibarquedoor && dicurrent == dicarruec )
		this.PlayGameSound( "fx/vulcan/oldelevator_clack" );
	}
	//! Called when closed
	event void OnClose()
	{
	if ( this == dibarquedoor )
		{
		this.PlayGameSound( "fx/door_close01" );
		this.PlayGameSound( "fx/door_close01" );
		}
	}
};

//! can cast any projectile in the game.
class ProjectileCaster extends CPart
{
	//! Starts casting projectiles
	proto native void Start();
	//! Stops casting projectiles
	proto native void Stop();

	//! Fire Once (suitable only for some projectiles) (at caster coordinates)
	proto native void FireOnce();

	//! Emit once (suitable only for some projectiles) (at specified coordinates)
	proto native void EmitOnceAt( vector vCoords );

	//! Called when projectile emited
	event void OnProjectileEmit()
	{
	}

};

//! Construction Shield
class ConstructionShield extends CPart
{
	reference int Color;
	reference int AlphaTest;
};
/* @} */
class func_terrain extends NonInteractiveEntity
{
};

class FuncGroup extends GameEntity
{
};
/** \addtogroup MissionsAPI */
/**@{*/
//! can be inserted into the map via WB
class CutscenePoint extends NullObject
{
	//! play normal cutscene
	proto native bool Play();
	//! play cutscene ingame
	proto native bool PlayIngame();
	//! stop cutscene
	proto native void Stop();
	//! is cutscene playing?
	proto native bool IsPlaying();
	//! add camera target to any time
	proto native bool SetCameraTargetKey(int key, string target);
	//! set DOF effect intensity
	proto native void SetDOFPower(float power);
	//! set DOF effect focus distance from camera (in camera look axis)
	proto native void SetDOFFocus(float depth, float time);
	//! get DOF focus value. see SetDOFFocus
	proto native float GetDOFFocus();
	//! Set SmartGamma custom material
	proto native void SetSmartGammaMaterial( string sMaterial );
	//! Set SmartGamma current LERP
	proto native void SetSmartGammaLerp( float fLerp );
	//! set DOF near effect intensity
	proto native void SetDOFNearPower(float power);
	//! set DOF material
	proto native void SetDOFMaterial(string material);
	//! set default DOF material
	proto native void SetDefaultDOFMaterial();
	//! set SSAO material
	proto native void SetSSAOMaterial(string material);
	//! set default SSAO material
	proto native void SetDefaultSSAOMaterial();
	//! set SSAO material
	proto native void SetFilmGrainMaterial(string material);
	//! set default SSAO material
	proto native void SetDefaultFilmGrainMaterial();
	//! set camera near plane. this overwrites value from camera animation (if any). Set 0 to enable animated default
	proto native void SetCameraNearPlane(float nearPlane);
	//! set camera far plane. with 0 means deafult value (160000 at this time)
	proto native void SetCameraFarPlane(float farPlane);

	//! timeline script event
	event void OnTimelineEvent(int event)
	{
	}
	//! on cutscene start or restart ("play scene again" during playing)
	event void OnStart()
	{
	}
	//! deprecated. duplicate to OnEnd
	event void OnStop(bool finished)
	{
	}
	//! on cutscene end (fully watched or skipped). pair with OnStart. to every OnStart, OnEnd is always called
	event void OnEnd(bool finished)
	{
	}
	//! called after cutscene OnEnd(), but only if cutscene is fully watched
	event void OnFinish()
	{
	}
	//! called after cutscene OnEnd(), but only if cutscene is interrupted before finish
	event void OnSkip()
	{
	}
};
/* @} */

class CutsceneActorPoint extends NullObject
{
};

//! Sprite
class Sprite extends NonInteractiveEntity
{
	reference int Color;
	reference float AlphaBias;
	reference int AlphaTest;
};


class ParticleAncestor extends NonInteractiveEntity
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! inserted into the map via WB
class ParticleEffect extends ParticleAncestor
{

	//! Start particle now
	proto native bool Start();

	//! Stop particle now
	proto native bool Stop();

	//! On emiting stop
	event void OnStop()
	{
	}

};
/**@}*/

class HitEffect extends ParticleAncestor
{
};

class ParticleAmbient extends NonInteractiveEntity
{
};

class HeatHaze extends NonInteractiveEntity
{
};

class CameraEffect extends NonInteractiveEntity
{
};
class CameraSpriteRefract extends NonInteractiveEntity
{
};

class SModel extends NonInteractiveEntity
{
};

class StaticOccluder extends NonInteractiveEntity
{
};

class InteriorOccluder extends NonInteractiveEntity
{
};

class StaticModel extends SModel
{
};

class StaticExpletory extends SModel
{
};

class StaticModelNoObstacle extends SModel
{
};

class RModel extends SModel
{
};

class RepaintModel extends RModel
{
};

class Flag extends RModel
{
};

class ClothModel extends SModel
{
};

//! StaticModelDisplay
class StaticModelDisplay extends SModel
{
	reference float mat01_shiftOffsetU;
	reference float mat01_shiftOffsetV;
	reference float mat01_scaleCenterU;
	reference float mat01_scaleCenterV;
	reference float mat01_scaleU;
	reference float mat01_scaleV;
	reference float mat01_rotateAngle;
	reference float mat01_rotateCenterU;
	reference float mat01_rotateCenterV;
	reference float mat01_color[4];
	
	reference float mat02_shiftOffsetU;
	reference float mat02_shiftOffsetV;
	reference float mat02_scaleCenterU;
	reference float mat02_scaleCenterV;
	reference float mat02_scaleU;
	reference float mat02_scaleV;
	reference float mat02_rotateAngle;
	reference float mat02_rotateCenterU;
	reference float mat02_rotateCenterV;
	reference float mat02_color[4];
	
	reference float mat03_shiftOffsetU;
	reference float mat03_shiftOffsetV;
	reference float mat03_scaleCenterU;
	reference float mat03_scaleCenterV;
	reference float mat03_scaleU;
	reference float mat03_scaleV;
	reference float mat03_rotateAngle;
	reference float mat03_rotateCenterU;
	reference float mat03_rotateCenterV;
	reference float mat03_color[4];
	
	reference float mat04_shiftOffsetU;
	reference float mat04_shiftOffsetV;
	reference float mat04_scaleCenterU;
	reference float mat04_scaleCenterV;
	reference float mat04_scaleU;
	reference float mat04_scaleV;
	reference float mat04_rotateAngle;
	reference float mat04_rotateCenterU;
	reference float mat04_rotateCenterV;
	reference float mat04_color[4];
	
	reference float mat05_shiftOffsetU;
	reference float mat05_shiftOffsetV;
	reference float mat05_scaleCenterU;
	reference float mat05_scaleCenterV;
	reference float mat05_scaleU;
	reference float mat05_scaleV;
	reference float mat05_rotateAngle;
	reference float mat05_rotateCenterU;
	reference float mat05_rotateCenterV;
	reference float mat05_color[4];
	
	reference float mat06_shiftOffsetU;
	reference float mat06_shiftOffsetV;
	reference float mat06_scaleCenterU;
	reference float mat06_scaleCenterV;
	reference float mat06_scaleU;
	reference float mat06_scaleV;
	reference float mat06_rotateAngle;
	reference float mat06_rotateCenterU;
	reference float mat06_rotateCenterV;
	reference float mat06_color[4];
	
	reference float mat07_shiftOffsetU;
	reference float mat07_shiftOffsetV;
	reference float mat07_scaleCenterU;
	reference float mat07_scaleCenterV;
	reference float mat07_scaleU;
	reference float mat07_scaleV;
	reference float mat07_rotateAngle;
	reference float mat07_rotateCenterU;
	reference float mat07_rotateCenterV;
	reference float mat07_color[4];
	
	reference float mat08_shiftOffsetU;
	reference float mat08_shiftOffsetV;
	reference float mat08_scaleCenterU;
	reference float mat08_scaleCenterV;
	reference float mat08_scaleU;
	reference float mat08_scaleV;
	reference float mat08_rotateAngle;
	reference float mat08_rotateCenterU;
	reference float mat08_rotateCenterV;
	reference float mat08_color[4];
	
	reference float mat09_shiftOffsetU;
	reference float mat09_shiftOffsetV;
	reference float mat09_scaleCenterU;
	reference float mat09_scaleCenterV;
	reference float mat09_scaleU;
	reference float mat09_scaleV;
	reference float mat09_rotateAngle;
	reference float mat09_rotateCenterU;
	reference float mat09_rotateCenterV;
	reference float mat09_color[4];
	
	reference float mat10_shiftOffsetU;
	reference float mat10_shiftOffsetV;
	reference float mat10_scaleCenterU;
	reference float mat10_scaleCenterV;
	reference float mat10_scaleU;
	reference float mat10_scaleV;
	reference float mat10_rotateAngle;
	reference float mat10_rotateCenterU;
	reference float mat10_rotateCenterV;
	reference float mat10_color[4];
	
	reference float mat11_shiftOffsetU;
	reference float mat11_shiftOffsetV;
	reference float mat11_scaleCenterU;
	reference float mat11_scaleCenterV;
	reference float mat11_scaleU;
	reference float mat11_scaleV;
	reference float mat11_rotateAngle;
	reference float mat11_rotateCenterU;
	reference float mat11_rotateCenterV;
	reference float mat11_color[4];
	
	reference float mat12_shiftOffsetU;
	reference float mat12_shiftOffsetV;
	reference float mat12_scaleCenterU;
	reference float mat12_scaleCenterV;
	reference float mat12_scaleU;
	reference float mat12_scaleV;
	reference float mat12_rotateAngle;
	reference float mat12_rotateCenterU;
	reference float mat12_rotateCenterV;
	reference float mat12_color[4];
	
	reference float mat13_shiftOffsetU;
	reference float mat13_shiftOffsetV;
	reference float mat13_scaleCenterU;
	reference float mat13_scaleCenterV;
	reference float mat13_scaleU;
	reference float mat13_scaleV;
	reference float mat13_rotateAngle;
	reference float mat13_rotateCenterU;
	reference float mat13_rotateCenterV;
	reference float mat13_color[4];
	
	reference float mat14_shiftOffsetU;
	reference float mat14_shiftOffsetV;
	reference float mat14_scaleCenterU;
	reference float mat14_scaleCenterV;
	reference float mat14_scaleU;
	reference float mat14_scaleV;
	reference float mat14_rotateAngle;
	reference float mat14_rotateCenterU;
	reference float mat14_rotateCenterV;
	reference float mat14_color[4];
	
	reference float mat15_shiftOffsetU;
	reference float mat15_shiftOffsetV;
	reference float mat15_scaleCenterU;
	reference float mat15_scaleCenterV;
	reference float mat15_scaleU;
	reference float mat15_scaleV;
	reference float mat15_rotateAngle;
	reference float mat15_rotateCenterU;
	reference float mat15_rotateCenterV;
	reference float mat15_color[4];

	void StaticModelDisplay()
	{
		mat01_scaleU = 1;
		mat01_scaleV = 1;
		mat02_scaleU = 1;
		mat02_scaleV = 1;
		mat03_scaleU = 1;
		mat03_scaleV = 1;
		mat04_scaleU = 1;
		mat04_scaleV = 1;
		mat05_scaleU = 1;
		mat05_scaleV = 1;
		mat06_scaleU = 1;
		mat06_scaleV = 1;
		mat07_scaleU = 1;
		mat07_scaleV = 1;
		mat08_scaleU = 1;
		mat08_scaleV = 1;
		mat09_scaleU = 1;
		mat09_scaleV = 1;
		mat10_scaleU = 1;
		mat10_scaleV = 1;
		mat11_scaleU = 1;
		mat11_scaleV = 1;
		mat12_scaleU = 1;
		mat12_scaleV = 1;
		mat13_scaleU = 1;
		mat13_scaleV = 1;
		mat14_scaleU = 1;
		mat14_scaleV = 1;
		mat15_scaleU = 1;
		mat15_scaleV = 1;
		
		for(int n = 0; n < 4; n++)
		{
			mat01_color[n] = 1;
			mat02_color[n] = 1;
			mat03_color[n] = 1;
			mat04_color[n] = 1;
			mat05_color[n] = 1;
			mat06_color[n] = 1;
			mat07_color[n] = 1;
			mat08_color[n] = 1;
			mat09_color[n] = 1;
			mat10_color[n] = 1;
			mat11_color[n] = 1;
			mat12_color[n] = 1;
			mat13_color[n] = 1;
			mat14_color[n] = 1;
			mat15_color[n] = 1;
		}
	}
	
	proto native void SetState(string state);
	
	//! Can be called and used like the EOnTick event
	event void OnStateChange(string prevState, string newState);
};

class SuperEntity extends SModel
{
};

class CameraPoint extends NullObject
{
};

class TreeModel extends SModel
{
};

class BushModel extends SModel
{
};

class StaticVisualizer extends SModel
{
};

//! ForceAncestor
class ForceAncestor extends SModel
{
	//! activate the shiled
	proto native void Activate();
	//! drop the shield
	proto native void Deactivate();
};
class ForceField extends ForceAncestor
{
};
class ForceDamager extends ForceAncestor
{
};

//! WaterModel
class WaterModel extends SModel
{
	reference int Terrain;
};


/** \addtogroup StrategicAPI */
/* @{*/
//! is used for strategic AI navigation on island when fighting on non-loaded island
class IslandKeyPoint
{
	//! Get Island Key Point Name
	proto native owned string GetName();

	//! Get Island Key Point Position
	proto native vector GetPosition();

	//! Get Island Key Point Flags
	proto native int GetFlags();

	//! Kill All Units Around Island Key Point
	proto native void KillAllUnitsAround();

	//! Calculate Island Key Point Combat Power
	proto void CalculateCombatPower( float aaPower, float atPower, float aaDefense, float atDefense );

	//! Visualize Island Key Point As Path (on map)
	proto native void VisualizeAsPath( IslandKeyPoint prevPoint );
	//! Visualize Island Key Point As Fight (on map)
	proto native void VisualizeAsFight();
	//! Visualize Island Key Point As Captured (on map)
	proto native void VisualizeAsCaptured();
};
/* @} */

// some flags for IslandModel::GetNeighbors
const int FRIENDLY_ISLANDS = 1;
const int ENEMY_ISLANDS = 2;

/** \addtogroup StrategicAPI */
/**@{*/
//! island API
class IslandModel extends SModel
{
	reference int cColor;
	reference int eColor;

	//! Determine area index of island
	proto native int GetArea();
	//! Get ID in island list
	proto native int GetListID();
	//! How many times player visited this island
	proto native int GetVisitedCount();

	//! Force island type at runtime (island must be loaded)
	proto native void ForceType( int eType );

	//! Change island type
	proto native void SetType( int eType );
	//! Get current island type
	proto native int GetType();

	//! Correct way to change island type
	proto native void InitiateTypeChange( int eType );
	//! Correct way to change stockpile island
	proto native void InitiateStockpileMove();

	//! Start command center build using item type (ie CapsuleDefense - use direct item name from DB)
	proto native void BuildCommandCenter( CCPlayer pPlayer, int eBuildItem );
	//! Check if command center build in progress on this island
	proto native bool IsBuildingCommandCenter();
	//! Check if command center build in progress on this island (for player)
	proto native bool IsBuildingCommandCenterForPlayer();

	//! Assign island to given player, true if the IslandCaptured event has to be called
	proto native void AssignToPlayer( CCPlayer player, bool bCampaignEvent );

	//! Get area size
	proto native float GetAreaSize();

	//! Get total production power
	proto native float GetProductionTotal();
	//! Get total mining power
	proto native float GetResourcesTotal();

	//! Get reachable neighbors
	// this method returns 'maxCnt' (if there is enough) islands filtered by 'flags' which are sorted
	// by total distance required to sail from current Carrier's position to the island and then back
	// to stockpile
	proto void GetNeighbors( out IslandModel neighbors[], out int neignCnt, CCPlayer forPlayer, int maxCnt, int flags );

	//! get neighbors connected by island network lilnks
	proto void GetConnectedNeighbors( out IslandModel neighbors[], out int neighCount );

	/*!
	Set exact weather forecast
	three params : first float is which new weather to set, second is constant time of current weather and
	third is time of change to new weather, time is in seconds
	*/
	proto native void SetWeatherForecast( vector params);

	/*!
	Set exact weather forecast
	two params : first float is which new weather to set now, second is constant time of new weather,
	time is in seconds
	*/
	proto native void SetImmWeather( vector params);

	/*!
	Set exact wind angle in degrees
	two params : first float is new angle, second float is time of constant angle in seconds
	*/
	proto native void SetImmWindAngle( vector params);

	//! usr sky preset
	proto native void SetUserSkyPreset(string preset);

	//! Get Island Weather
	proto native float GetWeather();
	//! Get Island Wind Angle
	proto native float GetWindAngle();

	proto native IslandKeyPoint FindBestLandingSpot( vector from );
	proto native IslandKeyPoint GetFinalCCPoint();
	proto void GetScramblerPoints( out IslandKeyPoint points[], out int cnt );
	proto void GetKeyPointNeighbors( IslandKeyPoint point, out IslandKeyPoint neighs[], out int cnt );
	proto bool FindPathBetweenKeypoints( IslandKeyPoint from, IslandKeyPoint to, out IslandKeyPoint path[], out int length );

	//! Enable island usability in game
	proto native void EnableInGame( bool bEnable );
	//! Check if island is enabled in game
	proto native bool IsEnabledInGame();

	//! Enable island usability on map
	proto native void EnableOnMap( bool bEnable );
	//! Check if island is enabled on map
	proto native bool IsEnabledOnMap();

	//! When revealed to player
	proto native void OnRevealed();
	//! When concealed from player
	proto native void OnConcealed();

	//! Enable sail-to ability of island
	proto native void EnableForSail( bool bEnable );
	//! Check if island can be sailed to
	proto native bool IsEnabledForSail();

	//! assault start event
	proto native void OnAttackStart( CCPlayer pPlayer );
	//! assault stop event
	proto native void OnAttackStop( CCPlayer pPlayer );
	//! under attack?
	proto native bool IsUnderAttack();

	//! check whether this island is connected to it's player's network
	proto native bool IsConnected();
	//! check whether this island's slot is functional
	proto native bool IsProducing();

	//! Check if connected to stockpile
	proto native bool IsConnectedToStockpile();
	//! Check if has enemy island nearby
	proto native bool IsConnectedToEnemy();

	//! set/get island's repair rate multiplier
	proto native void SetRepairRateMultiplier( float fMult );
	proto native float GetRepairRateMultiplier();

	//! manually destroy island's buildings
	proto native void DestroyPier();
	proto native void DestroySlot();
	
	//! Set current unipost mission flags
	proto native void SetUniPostMission( int iMissionUniPost );
	//! Get current unipost mission flags
	proto native int GetUniPostMission();

	//! Set current unipost building type (table BuildingUniPost)
	proto native void SetUniPostType( int eBuildingUniPost, bool bCheat );
	//! Get current unipost building type (table BuildingUniPost)
	proto native int GetUniPostType();

	//! Set uniPost purpose globally on map
	proto native void SetUniPostPurpose( int iPurpose );
	//! Get uniPost purpose
	proto native int GetUniPostPurpose();

	//! Set uniPost usable by enemy globally on map
	proto native void SetUniPostUsableByEnemy( bool bUsable );
	//! Get uniPost complementation progress 0.0 .. 1.0
	proto native float GetUniPostProgress();
	//! Get active UniPost
	proto native UniPost GetActiveUniPost();

	//! Get UniPost count
	proto native int GetUniPostCount();
	//! Get UniPost
	proto native UniPost GetUniPost( int iIndex );

	//! Restore island UniPost settings
	proto native void UniPostRestore();
	//! Clear unipost settings
	proto native void UniPostClear();
	
	//! Check if any unipost mission is set
	proto native bool IsUniPostMission();

	//! Get time mark for ownership (mission time)
	proto native void SetOwnershipMark( float fTimeMark );
	//! Set time mark for ownership (mission time)
	proto native float GetOwnershipMark();

	//! Returns climate type (table IslandClimate)
	proto native int GetClimateType();

	//! Distribute island to position
	proto native void Distribute( float fX, float fY );

};
/**@}*/

//! Projectile
class Projectile extends NonInteractiveEntity
{

	//! Set target repair zone
	proto native void SetTargetRepairZone( int iRepairZone );

};

class ProjectileRocket extends Projectile
{
};

class ProjectileLaser extends Projectile
{
};

class ProjectileBullet extends Projectile
{
};

class ProjectileTracer extends Projectile
{
};

class ProjectileDecoy extends Projectile
{
};

//! Item
class Item extends CPart
{

	//! Get current projectile type
	proto native int GetProjectileType();

};

class Weapon extends Item
{
};

class Shield extends Item
{
};

class GearAncestor extends Item
{
};

class Gear extends GearAncestor
{
};

class MantaHook extends GearAncestor
{
};

class WalrusCapsule extends GearAncestor
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! ancestor for CC, SlotBuilding, UniPost and others...
class Building extends InteractiveEntity
{
	//! check if we can build on this one
	proto native bool IsConstructionSite();

	//! Initiate build process
	proto native void Build( CCPlayer pPlayer );

	//! Get building type
	proto native int GetBuildingType();

	//! If building is being repaired
	proto native bool IsUnderContruction();

	/*!	\name Events	*/
	//@{

	//! built from construction site
	event void OnBuilt()
	{
	}

	//! just been hacked
	event void OnHack()
	{
	}
	
	//! Players build/ hack attempt failed for specific reason
	event void OnBuildFail( int iReason )
	{
	}


	//! Called when build shield is up during assembly process
	event void OnBuildShieldRaised()
	{
	}

	//@}
};
/**@}*/

/** \addtogroup MissionsAPI */
/**@{*/
//! basic unit entity - for handling units from carriers (be carefull with units which can dock )
class EntityBasic
{
	//! Return occupied slot in Carrier (if not carrier unit, result == -1)
	proto native int GetSlot();

	//! Get Health
	proto native float GetHealth();
	//! Set Health
	proto native void SetHealth( float health );
	//! Get Fuel
	proto native float GetFuel();

	//! Check if unit is active
	proto native bool IsActive();
	//! Check if unit is dummy (cannot be controlled)
	proto native bool IsDummy();
	//! Check if unit is destroyed
	proto native bool IsDestroyed();
	//! Check if unit is deployable
	proto native bool IsDeployable();
	//! Check if unit is outside the Carrrier
	proto native bool IsFullyDeployed();

	//! Request deploy nextframe
	proto native void RequestDeploy();

	//! clear all equip
	proto native void ClearAllEquipment();
	//! same but for generic equip (only AI uses this)
	proto native void ClearAllEquipmentGeneric();

	//! Get equipment count
	proto native int GetEquipmentCount();

	//! Add item for safe equip
	proto native void SafeSetEquipment( int eDBEnum );
	//! Mark all items to be cleared
	proto native void SafeClearEquipment();
	//! Process safe equip/ clear
	proto native void SafeCommitEquipment();

	//! Set equipment item (automatically to particular slot and solve storage)
	proto native void SetEquipment( int eDBEnum );
	//! Set equipment item (DO NOT USE FROM AI UPDATE!)
	proto native void SetEquipmentForce( int eDBEnum );
	//! Get equipment item in particular slot
	proto native int GetEquipment( int iIndex );

	//! Get equipment combat props
	proto void GetEquipCombatStats( float aaPower, float atPower, float aaDefense, float atDefense, float support, float healing );

	//! Get entity
	proto native InteractiveEntity GetEntity();
	
	//! Destroy unit
	proto native void Destroy();

	//! Determine if unit is manta
	proto native bool IsManta();
	//! Determine if unit is walrus
	proto native bool IsWalrus();
	//! Determine if unit is turret
	proto native bool IsTurret();

	//! Deploy instantly to specified coordinates
	proto native bool InstantDeploy( vector vCoords, float fAngle );
	//! Dock instantly back to carrier
	proto native bool InstantDock();
	proto native vector GetCenter();
	proto native CCPlayer GetPlayer();

	//! Check if carrier is docked at harbour
	proto native bool IsDocked(IslandModel island);
	//! stop carrier
	proto native void Stop(bool force = false);
	//! cruising?
	proto native bool IsCruising();

	
	//! calculate the distance unit will be able to travel
	// before it uses all of its fuel.
	proto native float CalculateTravelDistance();
};
/**@}*/

class IslandTactic extends WayPointBase
{
	int AllWalruses;
	int AllMantas;
	int AllCharacters;
	int AllUnits;

	int MantasToSpawn;
	int WalrusesToSpawn;
	int CharactersToSpawn;

	int MaxTacticUnits;

	int SpawnedWalruses;
	int SpawnedMantas;
	int SpawnedCharacters;

	int TotalPowerWB;
	int TotalPowerSpawn;
	string TacticName;

	float IslandForce;

	//TEST
	event void OnEnemyNear( InteractiveEntity enemy )
	{
	}

	//! Spawn particular type of unit
	proto native InteractiveEntity SpawnUnit( SpawnPoint pSpawnPoint, int iType );

	proto native int GetTotalPower();
	proto native int GetBackupPower();
	proto native int GetSpawnUnitTypes();
	proto native int GetSpawnAttackType();


	bool EquipSpawnedUnit( InteractiveEntity entity, int iType )
	{
		float islFcd =  IslandForce;
		int items[10];
		int itemsMedium = 2;
		int itemsHeavy = 4;

		if ( iType == 1 )
		{
			items[0] = 9532;//wWalrusMachineGun
			items[1] = 7532;//wWalrusGrenade
			items[2] = 9535;//wWalrusLaser
			items[3] = 9735;//wWalrusShell
			items[4] = 12081;//wWalrusAA
			items[5] = 9534;//wWalrusPlasma
			items[6] = 13193;//wWalrusHeat
			items[7] = 2;//wWalrusLauncher
			items[8] = 9736;//wWalrusLauncher
			items[9] = 2;//wWalrusFlak
		}

		if ( iType == 2 )
		{
			items[0] = 13202;//wMantaMachineGuns
			items[1] = 13202;//wMantaMachineGuns
			items[2] = 11158;//wMantaLasers
			items[3] = 11158;//wMantaLasers
			items[4] = 11158;//wMantaLasers
			items[5] = 9741;//wMantaGatlings
			items[6] = 6;//wMantaPlasmas
			items[7] = 6;//wMantaPlasmas
			items[8] = 9741;//wMantaGatlings
			items[9] = 9741;//wMantaGatlings
		}

		int r;
		EntityBasic entBasic = entity.GetAssembly();
		
		if ( IslandForce <= 0.6 )
		{
			//ISLAND IS WEAK - NO ARMOR
			if ( iType == 1 || iType == 2 )
			{
				r = rand( 0, itemsMedium );
				if ( r > 1 )
					r = 1;
				entBasic.SetEquipment( items[r] );
			}
		}
		else
		{
			if ( IslandForce <= 1.1 )
			{
				//ISLAND IS NORMAL - SOME UNITS CAN WEAR ARMOR
				r = rand( 0,10 );
				if ( r > 7 )
				{
					//armorWalrusMedium
					if ( iType == 1 )
						entBasic.SetEquipment( 9739 );

					//armorMantaMedium
					if ( iType == 2 )
						entBasic.SetEquipment( 9737 );

					//addItems for units with armor - walruses can has some better weapons with armor
					r = rand( 0, itemsMedium );
					entBasic.SetEquipment( items[r] );
				}
				else
				{

					//addItems for units without armor
					r = rand( 0, itemsMedium );
					entBasic.SetEquipment( items[r] );
				}
			}
			else
			{
				if ( IslandForce <= 1.6 )
				{
					//ISLAND STRONG - ALL UNITS HAS ARMOR, FEW OF THEM HEAVY
					r = rand( 0,10 );
					if ( r < 8 )
					{
						if ( iType == 1 )
						{
							//armorWalrusMedium
							entBasic.SetEquipment( 9739 );
						}

						if ( iType == 2 )
						{
							//armorMantaMedium
							entBasic.SetEquipment( 9737 );
							r = rand( 0, 10 );
							if ( r < 5 )
							{
								//wMantaRocket
								entBasic.SetEquipment( 1109 );
							}
						}
					}
					else
					{
						if ( iType == 1 )
						{
							//armorWalrusHeavy
							entBasic.SetEquipment( 9740 );
						}

						if ( iType == 2 )
						{
							//armorMantaHeavy
							entBasic.SetEquipment( 9738 );
							r = rand( 0, 10 );
							if ( r < 5 )
							{
								//wMantaRocket
								entBasic.SetEquipment( 1109 );
							}
						}
					}

					r = rand( itemsMedium, itemsHeavy );
					entBasic.SetEquipment( items[r] );
				}
				else
				{
					if ( IslandForce <= 2.1 )
					{
						//ISLAND VERY STRONG - ALL UNITS HAS ARMOR, FEW OF THEM MEDIUM
						r = rand( 0,10 );
						if ( r < 3 )
						{
							if ( iType == 1 )
							{
								//armorWalrusMedium
								entBasic.SetEquipment( 9739 );
							}

							if ( iType == 2 )
							{
								//armorMantaMedium
								entBasic.SetEquipment( 9737 );
								r = rand( 0,10 );
								if ( r < 7 )
								{
									//wMantaRocket
									entBasic.SetEquipment( 1109 );
								}
								else
								{
									//wMantaMissile
									entBasic.SetEquipment( 5 );
								}
								
								r = rand( 0, 10);
								if ( r <= 1  )
								{
									//upgradeMantaShield
									entBasic.SetEquipment( 11478 );
								}
							}
						}
						else
						{
							if ( iType == 1 )
							{
								//armorWalrusHeavy
								entBasic.SetEquipment( 9740 );
							}

							if ( iType == 2 )
							{
								//armorMantaHeavy
								entBasic.SetEquipment( 9738 );
								r = rand( 0,10 );
								if ( r < 7 )
								{
									//wMantaRocket
									entBasic.SetEquipment( 1109 );
								}
								else
								{
									//wMantaMissile
									entBasic.SetEquipment( 5 );
								}
								
								r = rand( 0, 10);
								if ( r <= 1  )
								{
									//upgradeMantaShield
									entBasic.SetEquipment( 11478 );
								}
							}
						}
						
						r = rand( itemsMedium, sizeof( items) );
						entBasic.SetEquipment( items[r] );
					}
					else
					{
						//ISLAND IS DEADLY - ALL UNITS HAS HEAVY ARMOR
						if ( iType == 1 )
							entBasic.SetEquipment( 9740 );

						if ( iType == 2 )
						{
							entBasic.SetEquipment( 9738 );
							r = rand( 0,10 );
							if ( r < 4 )
							{
								//wMantaRocket
								entBasic.SetEquipment( 1109 );
							}
							else
							{
								//wMantaMissile
								entBasic.SetEquipment( 5 );
							}
							
							r = rand( 0, 10);
							if ( r <= 2  )
							{
								//upgradeMantaShield
								entBasic.SetEquipment( 11478 );
							}
						}

						r = rand( itemsHeavy,  sizeof( items) );
						entBasic.SetEquipment( items[r] );
					}
				}
			}
		}
	}



	//! called when tactic should spawn initial island defense
	void OnTacticInit( int iOrigTotalPower, int iFreeWalrus, int iFreeManta, int iFreeCharacter, int iUsedDefense, int iUsedAA, int iUsedAT, int iUsedAL )
	{
		TacticName = GetName();
		TotalPowerSpawn = iOrigTotalPower;
		TotalPowerWB = GetTotalPower();
		AllWalruses = iFreeWalrus;
		AllMantas = iFreeManta;
		AllCharacters = iFreeCharacter;
		int rUnits;

		if ( TotalPowerWB > 0 && TotalPowerSpawn > 0 )
		{
			IslandForce = TotalPowerSpawn / TotalPowerWB;

			if ( IslandForce <= 0.6 )
			{
				rUnits = rand( -1, 2 );
				WalrusesToSpawn = 1 + rUnits;
				MantasToSpawn = 1 + rUnits;
				CharactersToSpawn = 4 + rUnits;
				MaxTacticUnits = 2;
			}
			else
			{
				if ( IslandForce <= 1.1 )
				{
					rUnits = rand( -1, 2 );
					WalrusesToSpawn = 2 + rUnits;
					MantasToSpawn = 2 + rUnits;
					CharactersToSpawn = 3 + rUnits;
					MaxTacticUnits = 2;
				}
				else
				{
					if ( IslandForce <= 1.6 )	
					{
						rUnits = rand( 0, 2 );
						WalrusesToSpawn = 3 + rUnits;
						MantasToSpawn = 3 + rUnits;
						CharactersToSpawn = 0;
						MaxTacticUnits = 3;
					}
					else
					{
						if ( IslandForce <= 2.1 )
						{
							rUnits = rand( 0, 3 );
							WalrusesToSpawn = 3 + rUnits;
							MantasToSpawn = 3 + rUnits;
							CharactersToSpawn = 0;
							MaxTacticUnits = 4;
						}
						else
						{
							rUnits = rand( 0, 2 );
							WalrusesToSpawn = 3 + rUnits;
							MantasToSpawn = 3 + rUnits;
							CharactersToSpawn = 0;
							MaxTacticUnits = 5;
						}
					}
				}
			}
		}
		else
		{
			CharactersToSpawn = 0;
			WalrusesToSpawn = 0;
			MantasToSpawn = 0;
		}

/*
		Print( TacticName );
		Print( MaxTacticUnits );

		Print( TacticName );
		Print( WalrusesToSpawn );
		Print( MantasToSpawn );
		Print( CharactersToSpawn );
		Print( "-------------------------------------------------------------------------" );
*/
	}
	
	//! called when tactic should spawn initial island defense
	void OnTacticSpawn( SpawnPoint sp, int iType, int iCount )
	{
		// 1 == walrus, 2 == manta, 3 == droid
		int r;
		int c;
		InteractiveEntity entity;

		int total;
		switch( iType )
		{
			case 1:
				for( c = 0; c < WalrusesToSpawn; c++ )
				{
					total = SpawnedWalruses + SpawnedMantas;
					if ( total < MaxTacticUnits )
					{
						if ( c < iCount )
						{
							entity = SpawnUnit( sp, iType );
							SpawnedWalruses++;
							EquipSpawnedUnit( entity, iType );
						}
					}
				}
				break;

			case 2:
				for( c = 0; c < MantasToSpawn; c++ )
				{
					if ( c < iCount )
					{
						total = SpawnedWalruses + SpawnedMantas;
						if ( total < MaxTacticUnits )
						{
							entity = SpawnUnit( sp, iType );
							SpawnedMantas++;
							EquipSpawnedUnit( entity, iType );
						}
					}
				}
				break;

			case 3:
				for( c = 0; c < CharactersToSpawn; c++ )
				{
					if ( c < iCount )
					{
						if ( SpawnedCharacters <= CharactersToSpawn )
						{
							entity = SpawnUnit( sp, iType );
							SpawnedCharacters++;
							//EquipSpawnedUnit( entity, iType );
						}
					}
				}
				break;
		}
	}
};

class GuardBehavior extends IslandTactic
{
};

class AmbushBehavior extends IslandTactic
{
};

class ReinforcementsBehavior extends IslandTactic
{
};

class StrikeBackBehavior extends IslandTactic
{
};

class IslandTacticBinding extends WayPointBase
{
};

// some defines for 'action' parameter in unit command calls:
const int REPLACE = 0;	// clear plan & insert goal
const int APPEND = 1;		// append at the end of plan
const int PUSH = 2;			// push at the beginning of plan

/** \addtogroup MissionsAPI */
/**@{*/
//! walrus, manta and characters
class Unit extends InteractiveEntity
{
	//! set forsage power (0 - 1.0)
	proto native void Forsage( float power );

	//! read remaining forsage (0 - 1.0)
	proto native float GetRemainingForsage();

	//! go to position
	proto native void GoTo( vector target, int action );

	//! stop & wait @ position
	proto native void Wait( float timeInSeconds, int action );

	/*! turn at position (if possible)
	action 0 == clear plan&append | action == 1 append | action == 2 push as first
	*/
	proto native bool TurnAt( vector pos, int action );

	//! dock to parent Carrier
	proto native void Dock( int action );

	//! OBSOLETE!!!! follow route
	proto native void FollowRoute( string name, bool slowly, int action );

	//! follow route
	proto native void Follow( Route r, int action );

	//! follow route but do not use pathfinder. kinda hacky
	proto native void FollowNoPathfinder( Route r, int action );

	//! attack target
	proto native void Attack( InteractiveEntity target, int action );

	//! assist ally
	proto native void Assist( InteractiveEntity target, int action );

	//! clear unit plan
	proto native void ClearPlan();

	//! enqueue unit undock
	proto native void EnqueueUndock();

	// enable/disable unit motion
	proto native void SuppressMotion( bool bSupress );
	proto native bool MotionSuppressed();

	//! get current group
	proto native GroupEntity GetGroup();
	
	//! Check if unit can move
	proto native bool CanMove();

};
/**@}*/

// known character gestures
const int GESTURE_RELOAD = 1;
const int GESTURE_DANGER = 2;
const int GESTURE_ENGAGE = 3;
const int GESTURE_STOP = 4;

//anim modes
const int AM_DEFAULT = 0;					//default anim mode (AI anim system)
const int AM_DESIGNER = 1;					//anim mode controlled by designers from script (normal version, all bones are animated from animation )
const int AM_DESIGNER_NO_MOVEVEMENT = 2;	//anim mode controlled by designers from script (version without animated skeleton root bone)

/** \addtogroup VehiclesAPI */
/**@{*/
//! Characters (also droids) API
class Character extends Unit
{
	//! set optional target for head and eyes
	proto native void SetLookTarget( string target, bool turnToTarget, bool headToTarget = true, bool instant = false  );
	
	//! force closed eyes. To open eyes use OpenEyes() method
	proto native void CloseEyes(float closingTime);
	
	//! open eyes after using CloseEyes() method
	proto native void OpenEyes(float openingTime);

	//! play gesture animation (see enum above)
	proto native void PlayGesture( int gest );

	//! set mode for animating. "animMode" can be one from AM_DEFAULT, AM_DESIGNER, AM_DESIGNER_NO_MOVEVEMENT (see comments)
	proto native void SetAnimMode( int animMode )

	//! play activation anim (this also switches the character to unusable state)
	//NB: USE FOR DROIDS ONLY!
	proto native void DroidActivate();
	proto native void DroidDeactivate();

	proto native void Footstep(vector pos);

	//! AI locked?
	proto native bool AILocked();

	//! careful mode API
	proto native void CarefulMode( bool bEnable );

	//animhooks
	void LFootstep()
	{
		vector mins, maxs;
		GetBoundBox(this, mins, maxs);
		vector center = GetOrigin(this) + Vector(0, 0, mins[2]);
		vector offset = VectorToParent(Vector(-16, 0, 0));
		Footstep(center + offset);
	}
	void RFootstep()
	{
		vector mins, maxs;
		GetBoundBox(this, mins, maxs);
		vector center = GetOrigin(this) + Vector(0, 0, mins[2]);
		vector offset = VectorToParent(Vector(16, 0, 0));
		Footstep(center + offset);
	}
	
	//! Called whenever player tries to use this item
	event void OnShieldDown( int iRechargesRemain )
	{
		if( IsSelected() )
		{
			PlayGameSound("Game/Missions/Other/CombatSuit/suit_regen_start");
			SetShieldEventIdle(2);
		}
	}
	//! Called whenever player tries to use this item
	event void OnShieldRegenerated()
	{
		if ( IsSelected() )
		{
			PlayGameSound("Game/Missions/Other/CombatSuit/suit_regen_done");
			SetShieldEventIdle(2);
		}
	}
	//! Called whenever player tries to use this item
	event void OnShieldHurt( float fHealthBefore, float fHealthAfter )
	{
		if( IsSelected() )
		{
			float h = fHealthBefore - fHealthAfter;
			if (h > 0.02 && h <= 0.04)
			{
				PlayGameSound("Game/Missions/Other/CombatSuit/suit_injury_minor");
			}

			if (h > 0.04 && h <= 0.1)
			{
				PlayGameSound("Game/Missions/Other/CombatSuit/suit_injury_medium");
			}

			if (h > 0.1)
			{
				PlayGameSound("Game/Missions/Other/CombatSuit/suit_injury_major");
			}
			SetShieldEventIdle(2.5);
		}
	}

	//! No movement override for script
	proto native void OverrideNoMovement( bool bEnable );
	//! No shoot override for script
	proto native void OverrideNoShoot( bool bEnable );
	//! No rotate override for script
	proto native void OverrideNoRotate( bool bEnable );

	reference bool CharAnimActive;
};
/**@}*/

class Vehicle extends Unit
{
};

//! Aircraft
class Aircraft extends Vehicle
{
	//! enable/disable automatic flight level adjustment
	proto native void AutoAdjustFlightLevel( bool bAdjust );
};

class GroundVehicle extends Vehicle
{
};

class NavalVehicle extends Vehicle
{
};

class DefenseUnit extends Vehicle
{
};

class AutonomousUnit extends Vehicle
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! CommandCenters are special buildings needs some more API
class CommandCenter extends Building
{
	//! build command center using an unit
	proto native void Build( InteractiveEntity builder );
	//! hack command center using an unit
	proto native void Hack( InteractiveEntity hacker );

	//! build command center using player
	proto native void BuildByPlayer( CCPlayer pPlayer );
	//! hack command center using  player
	proto native void HackByPlayer( CCPlayer pPlayer );

	/*!	\name Events	*/
	//@{

	//! captured from 'prevPlayer', current player can be obtained by calling GetPlayer() as usual
	event void OnCaptured( CCPlayer prevPlayer )
	{
	}

	//! Called when carrier deploys entity from carrier which might possibly hack or build CC
	event void OnBuilderDeploy( InteractiveEntity pEntity )
	{
	}


	//@}
};
/**@}*/

class Generic extends Building
{
};

/** \addtogroup MissionsAPI */
/**@{*/
//! SlotBuildings are Factory, Rafinery, HQ, but also the small buildings and garages
class SlotBuilding extends Building
{
	//! Enter deploy preparation state
	proto native void Deploy( int iSlot, int eEntityStyle );
	//! Enter dock preparation state
	proto native void Dock( int iSlot, InteractiveEntity pEntity );
	//! Check if entity can deploy right now
	proto native int IsEnabled( int iSlot );
	//! Check entity can dock
	proto native int IsWorking( int iSlot );


	//! Cause total mayhem :D
	proto native void CauseMayhem();

	//! Get parent tactic if assigned any
	proto native IslandTactic GetParentTactic();

	//! Process hack by a player
	void ProcessHack( CCPlayer hacker )
	{
		// cause mayhem among island units
//		if( GetBuildingType() == BuildingHeadQuarters )
//		{
//			CauseMayhem();

//			//TODO: make some island units assisting and such
//		}
		
//		// on hack, convert 80% of all island healers to hacker's side
//		if( GetBuildingType() == BuildingFactory )
//		{
//			ConvertHealers( hacker, true, true, 0.8 );
//			//TODO: give player an item
//		}
		
	}

	//! Callback after deploy finished
	event void OnDeploy( InteractiveEntity pEntity )
	{
	}
	//! Callback after dock finished
	event void OnDock( InteractiveEntity pEntity )
	{
	}

	//! On various game event
	event void OnGameEvent( string sEventName )
	{
		switch(sEventName)
		{
		case "mantadock":
			PlayGameSound("vehicles/carrier_manta_dock");
			break;
		case "mantadeploy":
			PlayGameSound("vehicles/carrier_manta_deploy");
			break;
		case "walrusdock":
			PlayGameSound("vehicles/carrier_walrus_dock");
			break;
		case "walrusdeploy":
			PlayGameSound("vehicles/carrier_walrus_deploy");
			break;
		}
	}

};
/**@}*/

/** \addtogroup MissionsAPI */
/**@{*/
//! UniPosts are used for generic missions on islands (sramblers etc...)
class UniPost extends Building
{

	//! Get order in chain
	proto native int GetOrder();

	//! Check if purpose of this UniPost is complemented
	proto native bool IsPurposeComplemented();
	//! Check if purpose of this UniPost is failed
	proto native bool IsPurposeFailed();

	//! Check if already uncovered
	proto native bool IsUncovered();
	//! Uncover
	proto native void Uncover( bool bUncover );

	//! Set purpose flags
	proto native void SetPurposeFlags( int iFlags );
	//! Clear purpose flags
	proto native void ClearPurposeFlags( int iFlags );

};
/**@}*/

class Walrus;
/** \addtogroup VehiclesAPI */
/**@{*/
//! Mantas API
class Manta extends Aircraft
{
	//! enable/disable turning when idle
	proto native void EnableIdleHoover( bool enable );
	
	//! Grab cargo
	proto native void GrabCargoInstant( GameEntity pCargo );
	//! Release cargo
	proto native void ReleaseCargo();

	//! Call AI to go near and grab target
	proto native void GoAndGrab( Walrus target, int action );

	//! disable animating by game
	proto native void DisableAnimator();
	
	//! When manta has cargo which can be released
	event void OnCanReleaseCargo( GameEntity pCargo )
	{
	}
	//! When manta is in position above cargo
	event void OnCanGrabCargo( GameEntity pCargo )
	{
	if ( this == diunit[dicurrentunit] && !dihookinuse[dicurrentunit] && diisfullydeployed[dicurrentunit] )
		{
		ditempval = 0;
		for ( int digrabloopb = 5; digrabloopb < 9; digrabloopb++ )
			{
			if ( pCargo == diunit[digrabloopb] )
				ditempval = digrabloopb;
			}
		if ( di_manta_auto_hook == 2 || di_manta_auto_hook == 1 && diautohook[dicurrentunit] )
			{
			if ( ditempval && diunitautorefuelling[ditempval] )
				{
				if ( dihasplan[ditempval] && !diisplansuspended[ditempval] || pCargo.GetFuel() < 1 || pCargo.GetHealth() < 1 || direfuelmode < 2 && this.GetFuel() < 1 || direfuelmode != 1 && direfuelmode != 3 && this.GetHealth() < 1 )
					return;
				}
			this.SetInvincibility( true );
			dihookinuse[dicurrentunit] = 5;
			this.GrabCargoInstant(pCargo);
			}
		}
	}
	//! When manta leaved cargo area and cannot grab it from where
	event void OnLeftCargoArea()
	{
	}

	//! On various game event
	event void OnGameEvent( string sEventName )
	{
		switch(sEventName)
		{
		case "beinglocked":
			PlayGameSound("fx/beep_locking");
			break;
		switch(sEventName)
		{
		case "used":
			PlayGameSound("fx/activator_used");
			break;
		case "usestart":
			PlayGameSound("fx/activator_usestart");
			break;
		case "usefinished":
			PlayGameSound("fx/activator_usefinished");
			break;
		case "usefail":
			PlayGameSound("fx/activator_usefail");
			break;
		case "usebreak":
			PlayGameSound("fx/activator_usefail");
			break;
		}
		}
	}

	event void OnShieldDown( int iRechargesRemain )
	{
		if( IsSelected() )
		{
			PlayGameSound("Game/Missions/Other/CombatSuit/suit_regen_start");
			SetShieldEventIdle(2);
		}
	}

	event void OnShieldRegenerated()
	{
		if ( IsSelected() )
		{
			PlayGameSound("Game/Missions/Other/CombatSuit/suit_regen_done");
			SetShieldEventIdle(2);
		}
	}

	event void OnShieldHurt( float fHealthBefore, float fHealthAfter )
	{
		if( IsSelected() )
		{
			float h = fHealthBefore - fHealthAfter;
			if (h > 0.02 && h <= 0.08)
				PlayGameSound("Game/Missions/Other/CombatSuit/suit_injury_minor");

			if (h > 0.08 && h <= 0.2)
				PlayGameSound("Game/Missions/Other/CombatSuit/suit_injury_medium");

			if (h > 0.2)
				PlayGameSound("Game/Missions/Other/CombatSuit/suit_injury_major");

			SetShieldEventIdle(2.5);
		}
	}
	
};
/**@}*/

//! Drone
class Drone extends Aircraft
{
	//! blow the drone
	proto native void Explode();
};

/** \addtogroup VehiclesAPI */
/**@{*/
//! Walruses API
class Walrus extends GroundVehicle
{

	//! On various game event
	event void OnGameEvent( string sEventName )
	{
		switch(sEventName)
		{
		case "actorload":
			PlayGameSound("characters/walrus_actorload");
			break;
		case "actordrop":
			PlayGameSound("characters/walrus_actordrop");
			break;
		case "actordropbaulk":
			PlayGameSound("characters/walrus_actornodrop");
			break;
		case "actordropwater":
			PlayGameSound("characters/walrus_actornodrop");
			break;
		case "actordropspeed":
			PlayGameSound("characters/walrus_actordropspd");
			break;
		}
	}

};
/**@}*/

/** \addtogroup VehiclesAPI */
/**@{*/
//! Carriers API
class Carrier extends NavalVehicle
{
	//! Retrieve number of docked units of specific type
	proto native int GetDockedUnitCnt( int type );

	//! Check result before calling DeployUnit()
	proto native bool CanDeployUnits();

	//! Any drone outside?
	proto native bool DronesOut();

	//! Launch defense drones
	proto native void LaunchDrones();

	//! Deploy unit at specified slot
	proto native void AssemblyDeploy( int slot );

	//! Invoke strategic AI
	// This will give the control over Carrier to strategic part of the game
	// which will probably move it to another island or whatever.
	// Use to make Carrier move from current action island.
	proto native void InvokeStrategicAI();

	//! Dock at given island
	proto native void DockToIsland( IslandModel island );

	//! Instant dock to nearest free dockpoint
	proto native void OnToDockPoint( IslandModel island );

	//! Assault current island
	proto native void LaunchAssault();

	//! Get current carrier telemetry in action coordinates
	proto native float GetCarrierTelemetry();

	//! Get carrier repair system data
	proto native InteractiveRepairZone GetLocalRepairZone( int eZone );

	//! Undock Drones
	proto native void DefenseDronesUndock();
	//! Dock drones
	proto native void DefenseDronesDock();

	//! Launch Decoy Flare
	proto native void LaunchDecoyFlare();
	//! Launch Hammer Head
	proto native void LaunchHammerHead( vector vCoords );

	//! Set minimal engines ratio (even when destroyed) (0.0 .. 1.0)
	proto native void SetMinimalEngineRatio( float fMinEngineRatio );

	//! On various game event
	event void OnGameEvent( string sEventName )
	{
		switch(sEventName)
		{
		case "grounded":
			PlayGameSound("vehicles/carrier_grounded");
			break;
		case "mantadock":
			PlayGameSound("vehicles/carrier_manta_dock");
			break;
		case "mantadeploy":
			PlayGameSound("vehicles/carrier_manta_deploy");
			break;
		case "walrusdock":
			PlayGameSound("vehicles/carrier_walrus_dock");
			break;
		case "walrusdeploy":
			PlayGameSound("vehicles/carrier_walrus_deploy");
			break;
		}
	}
	
};
/**@}*/

class Barque extends NavalVehicle
{
};


/** \addtogroup MissionsAPI */
/**@{*/
//! used to change turret type ( on islandForce dependence )
class Turret extends DefenseUnit
{
	//! Island defense type
	proto native int GetIslandDefenseType();
	//! Island defense level
	proto native int GetIslandDefenseLevel();

	//! Island defense type
	proto native void SetIslandDefenseType( int eIslandDefenseType );
	//! Island defense level
	proto native void SetIslandDefenseLevel( int eIslandDefenseLevel );

};
/**@}*/

class GravonDisplay extends AutonomousUnit
{
};

class CameraController extends InteractiveEntity
{
};

class MyBody extends GameEntity
{
	dJoint JointToDestroy;
	void ~MyBody()
	{
		if(JointToDestroy)
		{
			dJointDestroy(JointToDestroy);
			JointToDestroy = NULL;
		}
	}
}

PhysicsDragTool dragTool;

ScriptModule myScript;

class ActionCamera extends CameraController
{
	proto native CameraSpriteRefract GetCameraShield ();
/*
	MyBody body1, body2;

	event void EOnFrame(_entity other, int extra)
	{
		if(dragTool)
		{
			dragTool.Update();
			if(body1 && body2 && dragTool && dragTool.holdBody)
			{
				vector err = body1.CoordToLocal(GetOrigin(body2)) - "512 0 0";
				Print(err);
			}
		}
	
		if(KeyState(KC_O))
		{
			ClearKey(KC_O);

			if(myScript)
			{
				myScript.Release();
				myScript = NULL;
			}
			
			myScript = LoadScript(g_Script, "scripts/test.h", true);
			if(myScript)
				myScript.Call(NULL, "MyTest", this);
		}
	}

	void ActionCamera()
	{
		if(!dragTool)
		{
			dragTool = new PhysicsDragTool(this);
		}
	}
	
	void ~ActionCamera()
	{
		if(dragTool)
		{
			delete dragTool;
			dragTool = NULL;
		}
	}
  */
};

/** \addtogroup StrategicAPI */
/**@{*/
//! events is better to solve in campaign and scripts writing in WB
class World extends GameEntity
{
	void World()
	{
		SetEventMask(this, EV_SIMULATE);
	}

	void ~World()
	{
	}
	
	/*!	\name Events	*/
	//@{

	//! Called after all entites loaded and their OnLoad was called
	event void OnAfterLoad()
	{
	}

	//! captured an island from 'prevPlayer'
	event void OnIslandCaptured( CCPlayer prevPlayer )
	{
	}

	//! modal dialog closed
	event void OnModalResult( int dialogID, int result )
	{
	}

	event void OnCheat(int ID)
	{
	}
	
	//@}  

	//! save any necessary persistent data
	event void SaveCustomData()
	{
	}

	//! load those data back
	event void LoadCustomData()
	{
	}

	event void OnQueueActionStarted(QueueAction action)
	{
	}

	event void OnQueueActionFinished(QueueAction action)
	{
	}
	
	event void EOnSimulate(_entity other, float timeslice)
	{
		//need to call it every simulation step, to keep it stable
		if(dragTool)
		{
			dragTool.HandleDragging();
		}
	}
};
/**@}*/

class StrategyCamera extends CameraController
{
};

/** \addtogroup VehiclesAPI */
/**@{*/
//! Interactive entity repair zone
class InteractiveRepairZone
{
	//! Get hit zone count
	proto native int GetHitZoneCount();
	
	//! Get specific AI zone
	proto native int GetNativeHitZone( int iHitZone );

	proto native vector GetCenter();
	//! Get specific hit zone origin
	proto native vector GetHitZoneOrigin( int iHitZone );
	//! Get health for particular hit zone
	proto native float GetHitZoneHealth( int iHitZone );
	//! Get component 
	proto native CPart GetHitZoneComponent(int iHitZone);
		
	//! Check if zone destroyed
	proto native bool IsDestroyed();

	//! Get health
	proto native float GetHealth();

	//! Set health
	proto native void SetHealth( float fDamage, bool bSmoke );

//		EPRIORITY_LOW
//		EPRIORITY_MEDIUM
//		EPRIORITY_HIGH

	//! Set default priority from script
	proto native void SetDefaultPriority( int ePriority );

	//! Set current priority from script
	proto native void SetPriority( int ePriority );

	//! Enable repairs of this particular zone
	proto native void EnableRepairs( bool bEnable );
	//! Enable repairs of this particular zone
	proto native bool IsRepairEnabled();

}
/**@}*/

/** \addtogroup VehiclesAPI */
/**@{*/
//! Carrier's Repair system
class RepairSystem
{
	//! Enable or disable repair system functionality
	proto native void EnableRepairs( bool bEnable );
	//! Enable repairs of this particular zone
	proto native bool IsRepairEnabled();

};
/**@}*/


//TODO: move elsewhere?
bool IsNull( class& ref )
{
	class c = ref;
	if( c ) return false;
	else return true;
}

Unit Ref2Ptr( Unit& ref )
{
	Unit e = ref;
	return e;
}




// Common traveler support entity
//
// Desc: This entity should be used for parenting entities upon self for mission purposes

class TravelerSupport extends ScriptedEntity
{
};


/** \addtogroup MissionsAPI */
/**@{*/
// Interior <> Exterior traveler entity
//! This entity representes lift travel, including noise sounds, camera shake and proper action
class TravelerLift extends ScriptedEntity
{
	int m_iDirection; ///< Current direction (0 == indoor 1 == outdoor)
	
	float m_fLength; ///< Travel length
	float m_fProgress; ///< Travel progress

	float m_fShake; ///< Shake interval

	// check if traveling indoor
	bool IsTravelingInDoor()
	{
		if( m_iDirection == 0 )
			return true;
		return false;
	}

	// check if traveling outdoor
	bool IsTravelingOutDoor()
	{
		if( m_iDirection == 1 )
			return true;
		return false;
	}

	// is called when travel finished
	event void OnTravelBegin()
	{
	}
	// is called when travel finished
	event void OnTravelFinished()
	{
	}

	// is called when lift shakes
	event void OnLiftShake( InteractiveEntity pActor )
	{
		// default increase by 1 sec
		m_fShake = m_fShake + 1;
//		g_Game.OnCameraShake(walrusGatling,1);
	}

	// is called when lift traveling
	event void OnTravelProgress( InteractiveEntity pActor, float fTravelProgress )
	{
	}

	// solve lift travel
	void LiftTravel( InteractiveEntity pActor, float fTimeSlice )
	{
		m_fProgress = m_fProgress + fTimeSlice;
		if( m_fProgress >= m_fLength )
		{
			m_fProgress = m_fLength;
			OnTravelFinished();
		}
		else
		{
			if( m_fShake >= 0 )
			{
				m_fShake = m_fShake - fTimeSlice;
				if( m_fShake <= 0 )
				{
					OnLiftShake(pActor);
				}
			}
		}

		// indoor progress from 0 .. 1
		float progress = m_fProgress / m_fLength;
		if( IsTravelingOutDoor() )
			progress = 1 - progress;

		OnTravelProgress(pActor,progress);
	}

	// control sequence
	event void OnFrame( InteractiveEntity pActor, float fTimeSlice )
	{
		LiftTravel(pActor, fTimeSlice);
	}

	// reset stats
	void Reset()
	{
		m_fShake = 0;
		m_fLength = 0;
		m_fProgress = 0;

		DisableComplex();
	}

	// traveling indoor
	void SetTravelIndoor()
	{
		m_iDirection = 0;
		Reset();
	}
	// traveling outdoor
	void SetTravelOutdoor()
	{
		m_iDirection = 1;
		Reset();
	}

	// begin travel
	void Travel( float fLength )
	{
		m_fLength = fLength;
		m_fProgress = 0;
	
		OnTravelBegin();
		EnableComplex();
	}

	// initiate teleport
	void TeleportActor( Door2 pDoor1, Door2 pDoor2, Character pActor )
	{
		vector actorPos = pActor.GetPosition();
		vector localPos = pDoor1.GetLocalPos(actorPos);
		vector dstPos = pDoor2.GetGlobalPos(localPos);

		float a1 = pDoor1.GetHOrientation();
		float a2 = pDoor2.GetHOrientation();
		float a3 = pActor.GetHOrientation();
		
		float angle = a3 - a1 + a2;

		// move actor
		pActor.Teleport(dstPos,angle);

		// disable traveler
		DisableComplex();
	}
};
/**@}*/

// Common traveler snatcher entity
//
// Desc: This entity is meant for special game event handling on world
class TravelerSnatcher extends ScriptedEntity
{
};



